/**************************************************************************
 *
 *	project				 : VisiScript
 *
 *	copyright            : (C) 2002 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: E:/home/cvsroot/visiscript/codeedit.cpp,v $
 *
 *  $Revision: 1.3 $
 *
 *	$Log: codeedit.cpp,v $
 *	Revision 1.3  2005/12/03 18:49:29  Michael
 *	added code to handle main file menu item
 *	
 *	Revision 1.2  2005/11/07 21:29:32  Michael
 *	changed default language
 *	
 *
 ***************************************************************************/
/*********************************************************************************
 *																				 *
 * This file is part of the VisiScript package (a frontend for script languages) *
 *																				 *
 * Copyright (C) 2002 by Michael Neuroth.										 *
 *                                                                               *
 * This program is free software; you can redistribute it and/or modify			 *
 * it under the terms of the GNU General Public License as published by    		 *
 * the Free Software Foundation; either version 2 of the License, or             *
 * (at your option) any later version.                                           *
 *                                                                               *
 ********************************************************************************/

#include "codeedit.h"

#include "qextscintillalexer.h"

#include <qtabwidget.h>

// ************************************************************************

#define MAX_BUFFER_LENGTH 512

  // Hilfsfunktion fuer SplitPath()
  static void _InsertChar( char * sBuffer, char ch )
  {
	int nLen = strlen( sBuffer );
	sBuffer[ nLen ] = ch;
	sBuffer[ nLen+1 ] = 0;
  }

  // Hilfsfunktion fuer SplitPath()
  // Transformation durchfuehren "abc" --> "cba"
  static void _RotateString( char * sBuffer )
  {
	char sBuf[MAX_BUFFER_LENGTH];
	int nLen = strlen( sBuffer );
	for( int i=0; i<nLen; i++ )
	{
		sBuf[ nLen-i-1 ] = sBuffer[i];
	}
	sBuf[ nLen ] = 0;
	strcpy( sBuffer, sBuf );
  }

  bool SplitPath( const char * sPath, QString & sDrive, QString & sDir, QString & sFileName, QString & sExt )
  {
	char sDriveBuf[12];
	char sDirBuf[MAX_BUFFER_LENGTH];
	char sNameBuf[MAX_BUFFER_LENGTH];
	char sExtBuf[MAX_BUFFER_LENGTH];
#ifndef __linux__
	_splitpath( /*(CHAR_CAST)*/sPath, sDriveBuf, sDirBuf, sNameBuf, sExtBuf );
#else
	// Simuliere _splitpath fuer Linux...
	strcpy( sDriveBuf, "" );
	strcpy( sDirBuf, "" );
	strcpy( sNameBuf, "" );
	strcpy( sExtBuf, "" );
	// Pfad von hinten analysieren
	char sBuffer[MAX_BUFFER_LENGTH];
	strcpy( sBuffer, "" );
	bool bFoundExt = false;
	bool bFoundFile = false;
	for( int i=strlen( sPath )-1; i>=0; i-- )
	{
		// ist the path something like that: '/abc/def/'
		if( !bFoundExt && ((sPath[i]=='/') || (sPath[i]=='\\')) )
		{
			bFoundExt = true;
			bFoundFile = true;
			// means: no extention and no filename found !
			strcpy( sExtBuf, "" );			// bugfix 1.6.2004
			_RotateString( sBuffer );		// bugfix 1.6.2004
			strcpy( sNameBuf, sBuffer );
			strcpy( sBuffer, "" );
		}
		if( !bFoundExt && !bFoundFile && sPath[i]=='.' )
		{
			bFoundExt = true;
			_InsertChar( sBuffer, sPath[i] );
			_RotateString( sBuffer );
			strcpy( sExtBuf, sBuffer );
			strcpy( sBuffer, "" );
		}
		else if( !bFoundFile && ((sPath[i]=='/') || (sPath[i]=='\\')) )
		{
			bFoundFile = true;
			_RotateString( sBuffer );
			strcpy( sNameBuf, sBuffer );
			strcpy( sBuffer, "" );
			_InsertChar( sBuffer, sPath[i] );
		}
		else
		{
			_InsertChar( sBuffer, sPath[i] );
		}
	}
	_RotateString( sBuffer );
	if( bFoundFile )
	{
		strcpy( sDirBuf, sBuffer );
	}
	else
	{
		strcpy( sNameBuf, sBuffer );
	}
#endif
	sDrive = sDriveBuf;
	sDir = sDirBuf;
	sFileName = sNameBuf;
	sExt = sExtBuf;
	//cout << "SPLIT: " << sPath << " --> " << sDirBuf << " *** " << sNameBuf << " *** " << sExtBuf << endl;
	return true;
  }

static QString StripPath( const QString & sFileName )
{
	QString sRet = sFileName;

//#ifdef ZAURUS
	QString sDrive;
	QString sDir;
	QString sFileNameTemp;
	QString sExt;

	SplitPath( (const char *)sFileName, sDrive, sDir, sFileNameTemp, sExt );

	sRet = sFileNameTemp+sExt;
//#endif

	return sRet;
}

// ************************************************************************

MyQCodeEdit::MyQCodeEdit( QWidget * parent, const char * name, QTabWidget * pTabWidget, QWidget * pTab, QWidget * pMainWin )
: QextScintilla( parent, name ),
  m_pLexer( 0 ),
  m_pTabWidget( pTabWidget ),
  m_pTab( pTab ),
  m_bIsMainFile( false ),
  m_bChanged( false ),
  m_iLanguageID( /*LANG_MINSCRIPT*/2 ),
  m_bWasLastKeyReturn( false )
{
    connect( this, SIGNAL( textChanged() ), this, SLOT( sltChanged() ) );
	connect( this, SIGNAL( isShowing() ), pMainWin, SLOT( sltUpdateChilds() ) );
	connect( this, SIGNAL( sigTabWithSelectedTextPressed() ), pMainWin, SLOT( sltTabWithSelectedTextPressed() ) );
	connect( this, SIGNAL( sigShiftTabWithSelectedTextPressed() ), pMainWin, SLOT( sltShiftTabWithSelectedTextPressed() ) );
}

MyQCodeEdit::~MyQCodeEdit()
{
	delete m_pLexer;
}

bool MyQCodeEdit::hasMarkedTxt()
{
	return hasSelectedText();
}

QString MyQCodeEdit::markedTxt()
{
	return selectedText();
}

void MyQCodeEdit::setWordWrap( bool bValue )
{
	if( bValue )
	{
		SendScintilla(QextScintillaBase::SCI_SETWRAPMODE, QextScintillaBase::SC_WRAP_WORD );
	}
	else
	{
		SendScintilla(QextScintillaBase::SCI_SETWRAPMODE, (unsigned long)QextScintillaBase::SC_WRAP_NONE );
	}
}

void MyQCodeEdit::setOverwriteMode( bool bValue )
{
	SendScintilla(QextScintillaBase::SCI_SETOVERTYPE, (unsigned long)bValue );
}

bool MyQCodeEdit::overwriteMode()
{
	return (bool)SendScintilla(QextScintillaBase::SCI_GETOVERTYPE );
}

void MyQCodeEdit::setHorizScrollbar( bool bValue )
{
	SendScintilla(QextScintillaBase::SCI_SETHSCROLLBAR, (unsigned long)bValue );
}

void MyQCodeEdit::setEdited( bool bValue )
{
	setModified( bValue );

	if( bValue )
	{
		sltChanged();
	}
	else
	{
		sltNotChanged();
	}
}

bool MyQCodeEdit::edited()
{
	return isModified();
}

void MyQCodeEdit::setLanguageID( int iLanguageID )
{
	m_iLanguageID = iLanguageID;
}

int MyQCodeEdit::languageID()
{
	return m_iLanguageID;
}

void MyQCodeEdit::setLexer( QextScintillaLexer * pLexer, int iLanguageID )
{
	delete m_pLexer;

	m_pLexer = pLexer;

	setLanguageID( iLanguageID );

	QextScintilla::setLexer( m_pLexer );
}

void MyQCodeEdit::updateFontForLexer( const QFont & aFont )
{
	if( m_pLexer )
	{
		m_pLexer->setFont( aFont );
		m_pLexer->setDefaultFont( aFont );
		m_pLexer->refreshProperties();
		QextScintilla::setLexer( m_pLexer );
	}
}

void MyQCodeEdit::setFileName( const QString & sFileName )
{
	m_sFileName = sFileName;

	UpdateTabName();
}

QString MyQCodeEdit::fileName() const
{
	return m_sFileName;
}

void MyQCodeEdit::sltChanged()
{
	m_bChanged = true;

	UpdateTabName();
}

void MyQCodeEdit::sltNotChanged()
{
	m_bChanged = false;

	UpdateTabName();
}

void MyQCodeEdit::sltUpdate()
{
	UpdateTabName();
}

void MyQCodeEdit::show()
{
	QextScintilla::show();

	emit isShowing();
}

void MyQCodeEdit::UpdateTabName()
{
	if( m_pTabWidget )
	{
		QString sAddTemp;
		QString sFrontAddTemp;

		if( m_bChanged )
		{
			sAddTemp = " *";
		}
		if( m_bIsMainFile )
		{
			sFrontAddTemp = "# ";
		}

		m_pTabWidget->changeTab( m_pTab, sFrontAddTemp+StripPath( m_sFileName )+sAddTemp );
	}
}

/*
void MyQCodeEdit::setFont( const QFont & f )
{
	int style = STYLE_DEFAULT;
	SendScintilla(SCI_STYLESETFONT,style,f.family().latin1());
	SendScintilla(SCI_STYLESETSIZE,style,f.pointSize());
	SendScintilla(SCI_STYLESETBOLD,style,f.bold());
	SendScintilla(SCI_STYLESETITALIC,style,f.italic());
	SendScintilla(SCI_STYLESETUNDERLINE,style,f.underline());
}

void MyQCodeEdit::setHorizScrollbar( bool bVisible )
{
	SendScintilla(QextScintillaBase::SCI_SETHSCROLLBAR, (unsigned long)bVisible );
}

void MyQCodeEdit::setMaxWidth( int iCount )
{
	SendScintilla(QextScintillaBase::SCI_SETSCROLLWIDTH, (unsigned long)iCount );
}
*/

void MyQCodeEdit::keyPressEvent(QKeyEvent *ke)
{
	m_bWasLastKeyReturn = false;

	switch( ke->key() )
	{
		case Key_Return:
			m_bWasLastKeyReturn = true;
			break;
		case Key_Backtab:
			if( hasSelectedText() )
			{
				emit sigShiftTabWithSelectedTextPressed();
				return;
			}
			break;
		case Key_Tab:
			if( hasSelectedText() )
			{
				emit sigTabWithSelectedTextPressed();
				return;
			}
			break;
		default:
			m_bWasLastKeyReturn = false;
	}

	QextScintilla::keyPressEvent( ke );
}

bool MyQCodeEdit::isLastKeyReturn() const
{
	return m_bWasLastKeyReturn;
}

bool MyQCodeEdit::isMainFile() const
{
	return m_bIsMainFile;
}

void MyQCodeEdit::setMainFile(bool bValue)
{
	m_bIsMainFile = bValue;
}

