/**************************************************************************
 *
 *	project				 : VisiScript
 *
 *	copyright            : (C) 2002-2006 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: E:/home/cvsroot/visiscript/visiscript.cpp,v $
 *
 *  $Revision: 1.27 $
 *
 *	$Log: visiscript.cpp,v $
 *	Revision 1.27  2006/01/22 13:19:00  Michael
 *	patches for zaurus platform
 *	
 *	Revision 1.26  2006/01/22 12:31:41  Michael
 *	added new configuration flag: automatic load at start
 *	
 *	Revision 1.25  2006/01/12 23:30:41  Michael
 *	simpler png handling implemented
 *	
 *	Revision 1.24  2006/01/12 23:26:20  Michael
 *	'problems' with png from memory was bug in open last files algorithm!
 *	
 *	Revision 1.23  2006/01/12 22:37:58  Michael
 *	new menu items Close all, Run act. Bugfixes, some modal Dialoges will be reused (position and size are restored), inlining all icons in memory (bin2c)
 *	
 *	Revision 1.22  2006/01/10 22:10:07  Michael
 *	some bugfixes and new features implemented. save and reload actual opend files.
 *	
 *	Revision 1.21  2005/12/16 16:07:56  Michael
 *	removed line feed in find-in-files (for zaurus platform)
 *	
 *	Revision 1.20  2005/12/16 08:07:14  Michael
 *	implemented thread support for zaurus
 *	
 *	Revision 1.19  2005/12/15 08:46:22  Michael
 *	better icon support
 *	
 *	Revision 1.18  2005/12/14 22:46:12  Michael
 *	added support to handle arguments for scripts
 *	
 *	Revision 1.17  2005/12/07 22:06:17  Michael
 *	Bugfix: use "" for script-names with spaces in path
 *	
 *	Revision 1.16  2005/12/03 19:38:43  Michael
 *	do not allow closing tab-page, if a find-in-files is still runing for this output-window
 *	
 *	Revision 1.15  2005/12/03 19:00:57  Michael
 *	implemented find-in-files in second thread
 *	
 *	Revision 1.14  2005/12/03 18:52:52  Michael
 *	new menu item Mark as main, bugfix: search for english words "line" and "file" in error messages in output window
 *	
 *	Revision 1.13  2005/11/27 17:41:50  Michael
 *	Patches to compile under Zaurus
 *	
 *	Revision 1.12  2005/11/27 16:36:32  Michael
 *	improved compile support for all platforms and many bugfixes. using QFile instead of file. solves problem with zaurus c3100?
 *	
 *	Revision 1.11  2005/11/20 22:38:51  Michael
 *	small patch for linux desktop
 *	
 *	Revision 1.10  2005/11/20 19:02:06  Michael
 *	patches for the zaurus
 *	
 *	Revision 1.9  2005/11/20 16:47:57  Michael
 *	updated docu, bugfixes
 *	
 *	Revision 1.8  2005/11/14 16:27:11  Michael
 *	added filebrowser-class from ZFax application, pachtes for zaurus port, implemented goto-dialog for zaurus
 *	
 *	Revision 1.7  2005/11/12 12:12:07  Michael
 *	updated documentation and fixes for zaurus port
 *	
 *	Revision 1.6  2005/11/10 22:50:26  Michael
 *	added tooltips to tab-pages, bugfix in find (case sensitive works now), possibility to inline icons as xpm.
 *	
 *	Revision 1.5  2005/11/09 22:17:55  Michael
 *	fixed some problems to compile under linux, bugfix for double-click in output-window.
 *	
 *	Revision 1.4  2005/11/08 20:54:07  Michael
 *	updated documentation
 *	
 *	Revision 1.3  2005/11/07 21:41:15  Michael
 *	implemented new features for next release
 *	
 *	Revision 1.2  2005/11/07 20:23:32  Michael
 *	add icon for application
 *	
 *	Revision 1.1.1.1  2005/11/06 11:34:15  Michael
 *	initial checkin of version 0.3.2
 *	
 *
 ***************************************************************************/
/*********************************************************************************
 *																				 *
 * This file is part of the VisiScript package (a frontend for script languages) *
 *																				 *
 * Copyright (C) 2002-2006 by Michael Neuroth.								     *
 *                                                                               *
 * This program is free software; you can redistribute it and/or modify			 *
 * it under the terms of the GNU General Public License as published by    		 *
 * the Free Software Foundation; either version 2 of the License, or             *
 * (at your option) any later version.                                           *
 *                                                                               *
 ********************************************************************************/

/*
    testing zaurus on cross-platform:
	qvfb&
	visiscript -qws

	creating translations (destop):
	lupdate visiscript.pro  (creates/updates *.ts files)
	lrelease visiscript.pro	(creates *.qm files)

  	Changes:

	23.8.2003   Menu item "Word wrap", "Overwrite" (toggle),
    24.8.2003   Save Script-Extention in config file, Tabspace handling implemented
	30.8.2003   saving the last selected directory implemented
	 1.4.2004   fixes for Zaurus c860 (layout manager), flag for show startup file dialog implemented
	30.4.2004   porting QScintilla back to Qt 2.3.2 for embedded Qt
	 1.5.2004   using QScintilla for the script editor, many new editor features
    27.5.2004   bugfixes, new QScintills version perl lexer using, better configure dialog,
	            multiple document support, last files list, code folding,

	new features:
	  * syntax hightlighing
	  * find text
	  * replace text
	  * show line numbers
	  * autoindent
	  * tabs as tab or spaces
	  * show whitespaces
	  * Zoom in/out
	  * set ratio of output to script window in config dialog
	  * improve Import Dialog --> use layout manager, needed for c860
	  * bugfix: crash with popen and gnuplot? --> bugfix in buffer-length

	new (11.5.2004):
	  * Bugfix: CheckData() in sltClearScript() called
	  * Bugfix: Open... label changed
	  * Bugfix: Search backwards label changed
	new (14.5.2004):
	  * port to windows
	  * perl parser enabled
	  * restore output ratio after program start
	  * using QScintilla 1.3
	new (17.5.2004):
	  * configure dialog with tabs implemented
	  * code folding implemented (configure dialog and menu item)
	  * configure CR/LF modus
	  * enable undo grouping for replace all
	  * implemented some invisible actions for accelerated keys for the zaurus
	    (CTRL+B and CTRL+E)
	new (19.5.2004):
	  * last read files list implemented
	  * ported to QScintilla 1.3
	new (22.5.2004):
	  * calling refreshProperties() at lexer after language change
	  * working on multi-document support
	  * ok, cancel buttons for configure, find and replace dialog for not zaurus
	  * added accelerator keys for not zaurus
	  * save and load menu items improved for not zaurus
    new (23.5.2004):
	  * accelerators for non zaurus in *.qm files defined
	  * translation handled for non zaurus
    new (24.5.2004):
	  * working on multi-document support
	  * some small bugfixes (setFocus(), better text selection colors)
    new (25.5.2004):
	  * working on multi-document support --> ready !
    new (26.5.2004):
	  * working on multi-document support, patches for Zaurus
    new (27.5.2004):
	  * working on drag and drop support for not Zaurus
	  * split path for tab-name implemented
	  * document changed signal implemented
	new (29.5.2004):
	  * CheckData() splited in CheckActData() and CheckAllData()
	  * Bugfix: filename setting in open was wrong
	  * Bugfix: font update after first show does not work
	  * introduced zvisiscript for better seperation of Zaurus and desktop systems
	new (31.5.2004):
	  * add number for empty documents
	  * Bugfixes for new features
	released version 0.3.1

    new (1.6.2004)
	  * port to linux
	  * some small bugfixes (bad tab view, bug in SplitPath())
    new (2.6.2004)
	  * Bugfix: removed virtual void sltClearActDocument(); out of zvisiscript.*
	  * Bugfix: binding in configure dialog for command field added
	new (13.9.2004)
	  * Lisp support added: qextscintillalexerlisp.h, qextscintillalexerlisp.cxx, qscintialla.pro
	  * last search/replace string handling added (ini-file, combobox)
	  * showing message-box if search-string was not found
	  * Bugfix: wrong number of parameters in findFirst()
	  * Bugfix: modified flag handled now also handled in undo/redo
	  * added status-bar (only not zaurus plattforms/desktop)
    new (14.9.2004)
	  * show line and position in status bar (only destop)
	new (20.9.2004)
	  * using QProcess for executing scripts
	new (21.9.2004)
	  * interaction modus implemented (menu item and configure dialog extended)
	new (27.9.2004)
	  * brace matching implemented
	  * auto completion implemented (set "none", "0" or "-1" as file name to disable auto completion)
	  * optimized the source code colorizing after changing the selection of the language
	  * improved the qscintilla library (removed unused code), reduced code size of visiscript
	new (28.9.2004)
	  * Bugfix: fixed synchronization problem in overwrite modus (menu and state)
	  * Bugfix: improved close handling (is text changed?)
	  * implemented "cancel" in question for saving file
    new (6.11.2004)
	  * Bugfix: modified flag was changed for all documents when a new document was loaded
	  * Bugfix: modified flag was changed for all documents when the language is changed
	  * command to comment out and uncomment the selected text block
	  * implemented tab and shift-tab to indent the selected text block
	  * automatic clearing of output if flag is set (see configure dialog)
	  * save and restore the directory of the last saved file
    new (7.11.2004)
	  * implemented configuration of autocompletion threshold
    released version 0.3.2

	new (28.2.2005)
	  * port to qt_net / visual c++.net
	  * saving and restoring last windows position and size
	  * port to QScintilla 1.62
    new (29.4.2005)
	  * port to QScintilla 1.62-1.5.1
			- zqscintilla.pro aus original --> delete unused Lexers, add Lisp-Lexer, add backport files: qptrlist.h, qsettings.h, qsettings.cpp, qsettings_unix.cpp
			- KeyWords.cxx patchen (nicht genutzte Lexer loeschen!)
			- ACHTUNG: Settings fuer include fuer QScintilla anpassen !
			- config for qscintilla: CONFIG	= qt warn_off release staticlib thread
	  * using buildin Ruby and Lua Lexer
	  * new default value for m_iShowStartupFileDialog (don't show the dialog at startup)
	  * setUtf8 implemented
	  * implemented the saving of the external programs to execute the script (max 10 script interpreters at a time)
    new (30.10.2005)
	  * port to QScintilla 1.65-1.6
	  * using buildin Tex, batch, css, Makefile Lexer
	  * Goto... implemented, for fast jump to new position
	  * new configuration flag to handle chdir before executing script
      * handle double-click on output window-text: jump to position in script if possible
    new (2.11.2005)
	  * new configuration flags: bChangeActDir, bAutoSave
	  * if there are no changes in the script, the original script is used to execute 
	  * using resize splitter for edit- and output-widgets (not for Zaurus)
	  * implemented printing of the script (not for Zaurus)
    new (3.11.2005)
	  * implemented find in files
    new (4.11.2005)
	  * implemented regular expressions for find in files
	  * saving settings of the find-dialogs
    new (7.11.2005)
	  * Bugfix: sequence of language ids has to fit with definition of menu items !!!
	  * automatic language detection with file extension implemented
	  * using icon (only desktop)
    new (9.11.2005)
	  * fixes for linux desktop port (SuSE 9.2)
	  * fixes for find-in-files and doubleclick-in-output handling
    new (10.11.2005)
	  * bugfix: bad argument sequence in findFirst() call
	  * setting ok as default button in find-in-files dialog
	  * added tooltips to every tab page
	  * now possible: using inline xpm-icons
	  * saving installation path in string (better load of resources when started from command-line)
    new (11.11.2005)
	  * port to zaurus (qscintilla and visiscript)
    new (12.11.2005)
	  * improved resize handling of dialogs
	  * checked in patched sources
	  * started with InputDlg for zaurus
	  * working on double-click replace on zaurus --> mouseDoubleClick()-ueberladen funktioniert !
    new (14.11.2005)
	  * implemented reg-expr find-in-files and goto-dialog for zaurus
	  * using ZFax filebrowser as new import/export file dialog
	  * stated to simulate the doubleClicked()-event for the zaurus
	  * patched QScintilla to use the font from the font-configuration dialog --> fixed space also for comments !
    new (15.11.2005)
      * fixing some build-problems for all platforms
      * updated the translations
	  * patched filebrowser for destop platforms
    new (16.11.2005)
      * cleanup sourcecode
	  * bugfix: handling flag m_bSetUtf8 for writing files
	  * bugfix (zaurus): original FileSelector handling was NOT multi-document save !!!
	  * removed original FileSelector handling, using new ZFax-filebrowser as Open/Save dialog
    new (20.11.2005)
	  * cleanups and tests for all platforms
    released version 0.4.0

    new (22.11.2005)
	  * using QFile instead of fstream, better UTF-8 support added
	  * bugfix: disable directory-button in find-in-files dialog
	  * allowing files as command-line options for zaurus
	  * working on file-reading problem with C3000 and C3100
	  * reusing old import/export-file-dialog for imports and exports on zaurus platform
	  * improved language support for find-in-files
    new (23.11.2005)
	  * fixes to compile source-code without patchs for every platform
    new (27.11.2005)
      * some bugfixes for linux platform (QSettings)
	  * fixes to compile source-code without patchs for every platform
    released version 0.4.1

	new (2.12.2005)
	  * bugfix: try to find english words for line and file if localized text is not found (error-messages of languages are english!)
	  * bugfix: overloading accept() method in filebrowser to fix problem with ok-button under zaurus!
	  * new menu-item: Mark as main
	  * known 'Bug': in CR/LF modus "windows" the (old) tab-pages will be changed when reading a new file !
	  * implemented find-in-files in a second thread, new menu-item: Stop find in files
	new (3.12.2005)
	  * do not allow closing tab-page, if a find-in-files is still runing for this output-window
	new (7.12.2005)
	  * bugfix: if a script-name has spaces in path use "" to execute the script
    new (13.12.2005)
	  * allow arguments for script-execution in configuration-dialog
    new (14.12.2005)
      * working on RPM support, better installation support for linux
    new (15.12.2005)
	  * improved icon handling (better support for installation)
	  * implemented find-in-files in thread for zaurus
    new (16.12.2005)
	  * improved RPM support
    released version 0.4.2

    new (10.1.2006)
	  * Bugfix: set cursor in last line after end of find-in-files (only desktop)
	  * Bugfix: in function getLanguageIDFromExtension()
      * save file list (and cursor position and last active file no) when closing and reload files after startup of application
	  * inlining all images, than the visiscript application needs only the exe-file and the four dll-files (qscintilla.dll, qt-mtnc321.dll, msvcr71.dll, msvcp71.dll) to distribute !!! 
    new (11.1.2006)
	  * new menu-item: (only desktop-platform) Act. Run implemented (Shift+F5)
	  * Bugfix: modal dialog not destroyed for (goto-line and find-in-files dialogs)
	  * some modal Dialoges will be reused (so the size and position is saved between calls): find, replace and find-in-files
	  * save and restore the actual main script
    new (12.1.2006)
	  * new menu-item: close all windows
	  * using bin2c.exe to convert png to binary data, now no *.png files for icons are needed ! an other convertion tool is wx.tools.img2xpm.py
    new (22.1.2006)
	  * double menu item to clear a script removed
	  * setting the hompepage url as tooltip for the about dialog
	  * added new flag in config-dialog for auto load of files at startup
    released version 0.4.3

  TODOs and wishes for upcomming releases:
  ========================================  
    * maybe: use only one output-window ?
    * maybe: ask for update/reload of externally changed file
	* maybe: simulate accelerator-keys for find and other menu-items for zaurus (in MyQCodeEdit::keyPressEvent())
    * maybe: configuration-flag to toggle the read-only modus of the output-window 
    * maybe: save the last used find-in-files-directory and file-extension 
    * maybe: search-button for find-in-files-directory does not work for zaurus
    * maybe: showing source-code structur in a tree for a better navigation
    * maybe: use file-extension to select the right script-interpreter (.py --> python, .rb --> ruby)
    * maybe: create online help
    * maybe: implement find-files: to search only for files
    * maybe: workaround to implement automatic scrolling in the output-window for zaurus (Qt 2.3)
    * maybe: better mouse support for windows: double-click on a single word should select the word
	* maybe: add/remove marker, find/jump to marker

    * implement debugging-support for Python and other languages:
        create a new margin and use markerDefine(), markerAdd(), markerDelete() to create breakpoints
    
 debug a python script:

 > python -m pdb test.py <dbg.ses
 > python -m pdb <dbg.ses test.py 

 dbg.ses:
   b 4
   c
   args
   whatis x
   p x

 access on local and global variables: globals(), locals(), vars() or dir()

*/

#include "visiscript.h"

// for debugging
//#include <iostream>
#include <algorithm>
using namespace std;

#include "ofontselector.h"
#ifdef _with_zfax_filedialog
#include "filebrowser.h"
#endif
#ifdef _with_opie_filedialog
#include "ofileselector.h"
#include "ofiledialog.h"
#else
#ifdef ZAURUS
#include "filedialogs.h"
#endif
#endif

#include "mymledit.h"
#include "codeedit.h"
#include "qextscintillalexercpp.h"
#include "qextscintillalexercsharp.h"
#include "qextscintillalexerpython.h"
#include "qextscintillalexerjava.h"
#include "qextscintillalexerjavascript.h"
#include "qextscintillalexerperl.h"
#include "qextscintillalexeridl.h"
#include "qextscintillalexersql.h"
#include "qextscintillalexerhtml.h"
#include "qextscintillalexerlua.h"
#include "qextscintillalexerlisp.h"
#include "qextscintillalexerruby.h"
#include "qextscintillaapis.h"
#include "qextscintillalexermakefile.h"
#include "qextscintillalexerbash.h"
#include "qextscintillalexerbatch.h"
#include "qextscintillalexercss.h"
#include "qextscintillalexertex.h"
#ifndef ZAURUS
#include "qextscintillaprinter.h"
#endif

#include <stdlib.h>
#include <stdio.h>
//#include <fstream>
#ifdef __linux__ // ZAURUS
#include <unistd.h>
#include <sys/wait.h>
#else
#include <process.h>
#endif

//#include <direct.h>		// for chdir

using namespace std;

#ifdef ZAURUS
#include "FontDlg.h"
#include <qpe/qpeapplication.h>
#include <qpe/qpemenubar.h>
#include <qpe/qpetoolbar.h>
#include <qpe/resource.h>
#include <qpe/fileselector.h>
#include <qpe/config.h>

#define QPERESOURCE(x) Resource::loadPixmap(x),
#define QPERESOURCE3(x,y,z,w) Resource::loadPixmap(x),

#define NOT_ZAURUS(x) 0
#define REPLACE_ZAURUS(x,y) y

#define MAX_LAST_FILES				5
#define MAX_EXTERNAL_PROGRAMS		10		// how many external script processors can be added to the script configuration box ?
#define MAX_LAST_SEARCH_STRINGS		10

#else
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qstatusbar.h> 
#include <qapplication.h>
#include <qsettings.h>
#include <qfiledialog.h>
#include <qsplitter.h>

#define INLINE_IMAGES		// TEST, for smaller distribution of the application (only exe and dll's)

#define QPERESOURCE(x)
#ifndef INLINE_IMAGES
#define QPERESOURCE3(x,y,z,w) QPixmap(y),		// using new_ico
#else

#ifdef INLINE_IMAGES_OLD
#define QPERESOURCE3(x,y,z,w) QPixmap(z),
#include "inline_img/editdelete_ico.xpm"
#include "inline_img/play_ico.xpm"
#include "inline_img/new_ico.xpm"
#include "inline_img/forward_ico.xpm"
#include "inline_img/find_ico.xpm"
#include "inline_img/fileopen_ico.xpm"
#include "inline_img/visiscript_ico.xpm"
#else
#define QPERESOURCE3(x,y,z,w) loadFromMem(w,sizeof(w)),		// using new_png
static QPixmap loadFromMem( uchar * pImgBuffer, uint iLen )
{
	QPixmap aImg;
	aImg.loadFromData( pImgBuffer, iLen );
	return aImg;
}
// created with bin2c.cpp (http://homepage.ntlworld.com/rik.griffin/bin2c.html)
#include "editdelete_png.h"
#include "play_png.h"
#include "new_png.h"
#include "forward_png.h"
#include "find_png.h"
#include "fileopen_png.h"
#include "visiscript_png.h"
#endif
#endif

#define NOT_ZAURUS(x) (x)
#define REPLACE_ZAURUS(x,y) x

#define MAX_LAST_FILES				10
#define MAX_EXTERNAL_PROGRAMS		10
#define MAX_LAST_SEARCH_STRINGS		20

#endif

#include <qaction.h>
#include <qpushbutton.h>
#include <qmultilineedit.h>
//#include <qtextview.h>
#include <qfontdialog.h>
#include <qfont.h>
#include <qfontdatabase.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qwidgetstack.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qdialog.h>
#include <qmessagebox.h>
#include <qinputdialog.h>
#include <qclipboard.h>
#include <qtabwidget.h>
#include <qobjectlist.h> 
#include <qdir.h>
#include <qregexp.h>
#include <qtextstream.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qevent.h> 
#include <qthread.h>

#include "configuredlgimpl.h"
#include "findinfilesdlgimpl.h"
#include "SaveAsDlg.h"
#include "AboutDlg.h"
#ifdef ZAURUS
#include "FindDlg.h"
#include "ReplaceDlg.h"
#include "InputDlg.h"
#include "qprocess.h"	// use the implementation from Qt 3 as copied file
#include <pthread.h>
#else
#include "finddlgimpl.h"
#include "replacedlgimpl.h"
#include <qprocess.h>
#include <qkeysequence.h> 
#endif

#define _TEMP_SCRIPT_FILE		"_tsf.xyz"
#define _TEMP_ERR_FILE			"_err.txt"
#define _TEMP_ERR_FILE_PIPE		"2>_err.txt"
#define _TEMP_OUT_FILE			"_out.txt"
#define _TEMP_OUT_FILE_PIPE		"1>_out.txt"
#define _VISISCRIPT_NAME		"visiscript"

#define _OUTPUT_NAME			"pOutput"
#define _SCRIPT_NAME			"pScript"

#define _USER_EVENT_MSG			1001
#define _USER_EVENT_DONE		1002

const int c_iSmall = 8;
const int c_iMedium = 10;
const int c_iLarge = 12;

const int c_iMinOutput = 80;
const int c_iMaxOutput = 200;
const int c_iMinScript = 80;
const int c_iMaxScript = 40;

const int c_iMaxOutputRatio = 70;
const int c_iMinOutputRatio = 15;

#define LANG_NONE 		1
#define LANG_MINSCRIPT	2
#define LANG_CPP		3
#define LANG_CSHARP		4
#define LANG_JAVA		5
#define LANG_JAVASCRIPT	6
#define LANG_PYTHON		7
#define LANG_RUBY		8
#define LANG_PERL		9
#define LANG_PHP		10
#define LANG_LUA		11
#define LANG_LISP		12
#define LANG_TCL		13
#define LANG_IDL		14
#define LANG_SQL		15
#define LANG_HTML		16
#define LANG_CSS		17
#define LANG_TEX		18
#define LANG_MAKEFILE	19
#define LANG_BATCH		20
#define LANG_BASH		21
#define LANG_MAX_ID		LANG_BASH+1

//#define LANG_SCHEME		22

// forward declaration from main.cpp
QString getInstallationPath();

static int getLanguageIDFromExtension( const QString & strExt )
{
	QString s = strExt.lower();

	if( s==".cpp" || s==".cxx" || s==".c" || s==".cc" || s==".h" || s==".hpp" || s==".i" || s==".xpm" )
	{
		return LANG_CPP;
	}
	else if( s==".java" )
	{
		return LANG_JAVA;
	}
	else if( s==".js" )
	{
		return LANG_JAVASCRIPT;
	}
	else if( s==".cs" )
	{
		return LANG_CSHARP;
	}
	else if( s==".py" || s==".pyw" )
	{
		return LANG_PYTHON;
	}
	else if( s==".rb" )
	{
		return LANG_RUBY;
	}
	else if( s==".pl" )
	{
		return LANG_PERL;
	}
	else if( s==".php" )
	{
		return LANG_PHP;
	}
	else if( s==".lua" )
	{
		return LANG_LUA;
	}
	else if( s==".cl" || s==".el" || s==".lisp" )
	{
		return LANG_LISP;
	}
	else if( s==".tcl" )
	{
		return LANG_TCL;
	}
	else if( s==".idl" )
	{
		return LANG_IDL;
	}
	else if( s==".sql" )
	{
		return LANG_SQL;
	}
	else if( s==".html" || s==".htm" || s==".xml" || s==".dtd" || s==".ui" || s==".properties" || s==".project" )
	{
		return LANG_HTML;
	}
	else if( s==".css" )
	{
		return LANG_CSS;
	}
	else if( s==".tex" )
	{
		return LANG_TEX;
	}
	else if( s==".bat" || s==".txt" || s==".log" )
	{
		return LANG_BATCH;
	}
	else if( s==".sh" )
	{
		return LANG_BASH;
	}
	else if( s==".pro" )
	{
		return LANG_MAKEFILE;
	}

	return LANG_NONE;
}


// ************************************************************************

static int g_iDocCount = 1;

static int u_id = 1;
static int get_unique_id()
{
    return u_id++;
}

static QString stripPath( const QString & strFileName )
{
	QString strRet = strFileName;
	strRet.replace( QString("\\"), QString("/") );
	strRet.replace( QString("\""), QString("") );
	return strRet;
}

static bool ReadTextFile( const QString & sFullFileName, QString & sText, bool bUtf8 = false )
{
	QFile aFile( sFullFileName );

	if( !aFile.open(IO_ReadOnly | IO_Translate) )
	{
		return false;
	}
	else
	{
		QTextStream aInStream( &aFile );
		if( bUtf8 )
		{
			aInStream.setEncoding(QTextStream::UnicodeUTF8);
		}
		else
		{
			aInStream.setEncoding(QTextStream::Latin1);
		}

		sText = aInStream.read();

		return true;
	}
}

static bool WriteTextFile( const QString & sFullFileName, const QString & sText, bool bAppend = false, bool bUtf8 = false )
{
	QFile aFile( sFullFileName );

	if( !aFile.open( (bAppend ? IO_Append : IO_WriteOnly) | IO_Translate) )
	{
		return false;
	}
	else
	{
		QTextStream aOutStream( &aFile );
		if( bUtf8 )
		{
			aOutStream.setEncoding(QTextStream::UnicodeUTF8);
		}
		else
		{
			aOutStream.setEncoding(QTextStream::Latin1);
		}

		aOutStream << sText;

		return true;
	}
}

static bool ExistsInList( const QString & sName, QList<QString> & aList )
{
	for( unsigned int i=0; i<aList.count(); i++ )
	{
		if( *(aList.at( i )) == sName )
		{
			return true;
		}
	}

	return false;
}

static void PushToList( const QString & sName, QList<QString> & aList, unsigned int MAX=0 )
{
	if( !ExistsInList( sName, aList ) )
	{
		aList.append( new QString( sName ) );

		if( MAX>0 && aList.count()>MAX )
		{
			aList.removeFirst();
		}
	}
}

static QAction * createNewAction( const QString & strLabel, const QPixmap & aIcon, const QString & strMenuText, REPLACE_ZAURUS(QKeySequence,int) aKeySeq, QObject * pParent, const char * sName ){
#ifdef ZAURUS
	return new QAction( strLabel, aIcon, strMenuText, aKeySeq, pParent, sName );
#else
	return new QAction( aIcon, strLabel, aKeySeq, pParent, sName );
#endif
}

static QAction * createNewAction( const QString & strLabel, const QString & strMenuText, REPLACE_ZAURUS(QKeySequence,int) aKeySeq, QObject * pParent, const char * sName ){
#ifdef ZAURUS
	return new QAction( strLabel, strMenuText, aKeySeq, pParent, sName );
#else
	return new QAction( strLabel, aKeySeq, pParent, sName );
#endif
}


// ************************************************************************
/*
 *  Constructs a VisiScript which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 */
VisiScript::VisiScript( QWidget* parent,  const char* name, WFlags fl )
    : QMainWindow( parent, name, fl ),
	  m_pTabWidget( 0 ),
	  m_sScriptCmd( "minscript" ),
	  //m_sScriptArguments( "" ),
	  m_sFontName( "fixed" ),
	  m_sFontStyle( "normal" ),
	  m_iFontSize( c_iMedium ),
	  m_iWordWrap( 0 ),
	  m_iOverwrite( 0 ),
	  m_iSpacesForTabs( 0 ),
	  m_iShowWhitespaces( 0 ),
	  m_iShowLineNumbers( 1 ),
	  m_iAutoIndent( 0 ),
	  m_iCodeFolding( 0 ),
	  m_iTabSpace( 4 ),
	  m_iEolMode( 
#ifdef _MSC_VER
	    QextScintilla::EolWindows 
#else
		QextScintilla::EolUnix 
#endif
		),
	  m_sExeclCmd( "/bin/bash" ),
	  m_iExecuteMode( EXECUTE_MODE_POPEN ),
	  m_iShowStartupFileDialog( 0 ),
	  m_iOutputRatio( 30 ),
	  m_sSearchFiles( "*.py *.pl *.rb *.h *.cpp *.c *.java *.txt" ),
	  m_sSearchDir( "." ),
	  m_bAutoFileLoad( true ),
	  m_bIsReplace( false ),
	  m_bRegularExpr( false ),
	  m_bWildcard( false ),
	  m_bCaseSensitive( false ),
	  m_bOnlyWholeWords( false ),
	  m_bSearchBackward( false ),
	  m_bSearchSubDirectories( true ),
	  m_bReplaceAll( false ),
	  m_bInteractiveModus( false ),
	  m_bChangeActDir( true ),
	  m_bAutoSave( false ),
	  m_bBraceMatching( false ),
	  m_bAutoClearOutput( false ),
	  m_bSetUtf8( false ),
	  m_iActLine( 0 ),
	  m_iActPos( 0 ),
	  m_iAutoCompletionThresh( 2 ),
	  m_pAutoCompletionAPI( 0 ),
	  m_iLastMainScriptIndex( 0 ),
	  m_iLastActFileIndex( 0 ),
	  c_strLineTag( tr("line ") ),
	  c_strFileTag( tr("file ") ),
	  c_strLineTagOrg( "line " ),
	  c_strFileTagOrg( "file " ),
	  m_pFindDlg( 0 ),
	  m_pReplaceDlg( 0 ),
	  m_pFindInFilesDlg( 0 )
#ifdef _with_thread_search
	  , m_pFindThread( 0 )
	  , m_pFindOutputWindow( 0 )
	  , m_iCount( 0 )
	  , m_iFoundFileCount( 0 )
      , m_iTotalFileCount( 0 )
	  , m_bStopFlag( false )
#endif
#ifndef ZAURUS
	  , m_iLastPosX(5),
	  m_iLastPosY(5),
	  m_iLastPosDX(800),
	  m_iLastPosDY(600)
#endif
{
	loadInit();

	m_aLastCursorPos.setAutoDelete( TRUE );

	m_aLastFilesList.setAutoDelete( TRUE );
    m_aLastOpendFilesList.setAutoDelete( TRUE );
	m_aExternalProgsList.setAutoDelete( TRUE );
	m_aLastSearchStringList.setAutoDelete( TRUE );
	m_aLastReplaceStringList.setAutoDelete( TRUE );
	m_aLastSearchFilesList.setAutoDelete( TRUE );
	m_aLastSearchDirList.setAutoDelete( TRUE );
	m_aLastScriptArgumentsList.setAutoDelete( TRUE );

    setToolBarsMovable( FALSE );

	// *************************************************************
	// *************************************************************

#ifdef ZAURUS
    m_pMenu = new QPEToolBar( this );
    m_pMenuBar = new QPEMenuBar( m_pMenu );
    m_pButtonBar = new QPEToolBar( this );

    m_pMenu->setHorizontalStretchable( TRUE );
#else
#ifdef _MSC_VER
    m_pMenu = new QToolBar( this );
    m_pMenuBar = new QMenuBar( m_pMenu );
    m_pButtonBar = m_pMenu; //new QToolBar( this );
#else
	//m_pMenuBar = new QMenuBar( this ); //menuBar();
	m_pMenuBar = menuBar();
    m_pButtonBar = new QToolBar( this );
#endif

#ifdef INLINE_IMAGES
#ifdef INLINE_IMAGES_OLD
	QPixmap aIcon( visiscript_ico );
#else
	QPixmap aIcon = loadFromMem((uchar *)visiscript_png,sizeof(visiscript_png));
	//aIcon.loadFromData((uchar *)visiscript_png,sizeof(visiscript_png));
#endif
#else
	QPixmap aIcon( getInstallationPath()+QString("inline_img/Visiscript.png") );
	//QPixmap aIcon( getInstallationPath()+QString("inline_img/Visiscript16x16.png") );
#endif
	setIcon( aIcon );

	statusBar();
	m_pStatusBarMsg = new QLabel( statusBar() );
	statusBar()->addWidget( m_pStatusBarMsg/*, 5, FALSE*/ );

	setAcceptDrops( TRUE );

	//m_pMenu->setHorizontalStretchable( TRUE );
#endif

    m_pFile = new QPopupMenu( this );
    m_pEdit = new QPopupMenu( this );
    m_pView = new QPopupMenu( this );
	m_pLanguages = new QPopupMenu( m_pView );

    connect( m_pFile, SIGNAL( aboutToShow() ), this, SLOT( sltFileAboutToShow() ) );
    connect( m_pEdit, SIGNAL( aboutToShow() ), this, SLOT( sltEditAboutToShow() ) );
    connect( m_pView, SIGNAL( aboutToShow() ), this, SLOT( sltViewAboutToShow() ) );
    connect( m_pLanguages, SIGNAL( aboutToShow() ), this, SLOT( sltLanguagesAboutToShow() ) );
    connect( m_pLanguages, SIGNAL( activated(int) ), this, SLOT( sltLanguageActivated(int) ) );

    m_pMenuBar->insertItem( tr( "File" ), m_pFile );
    m_pMenuBar->insertItem( tr( "Edit" ), m_pEdit );
    m_pMenuBar->insertItem( tr( "View" ), m_pView );

	// ++++++++++++++++++++++++++++++++++++++++++

    QAction * aAction = createNewAction( tr( "New" ), /*QPERESOURCE( "new" )*/QPERESOURCE3("new",getInstallationPath()+QString("inline_img/new.png"),new_ico,new_png) QString::null, NOT_ZAURUS( CTRL+Key_N ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileNew() ) );
/**/    aAction->addTo( m_pButtonBar );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Clear" ), QString::null, NOT_ZAURUS( ALT+Key_C ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltClearActDocument() ) );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Close" ), QString::null, NOT_ZAURUS( ALT+Key_X ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltCloseActDocument() ) );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Close all" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltCloseAllDocuments() ) );
    aAction->addTo( m_pFile );

	m_pFile->insertSeparator();

    aAction = createNewAction( tr( "Open..." ), QPERESOURCE3("fileopen",getInstallationPath()+QString("inline_img/fileopen.png"),fileopen_ico,fileopen_png) QString::null, NOT_ZAURUS( CTRL+Key_O ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileOpen() ) );
/**/    aAction->addTo( m_pButtonBar );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Save" ), /*Resource::loadPixmap( "filesave" ),*/ QString::null, NOT_ZAURUS( CTRL+Key_S ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileSave() ) );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Save as..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileSaveAs() ) );
    aAction->addTo( m_pFile );

	m_pFile->insertSeparator();

    aAction = createNewAction( tr( "Import..." ), QString::null, NOT_ZAURUS( CTRL+Key_I ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileImport() ) );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Export..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileExport() ) );
    aAction->addTo( m_pFile );

    aAction = createNewAction( tr( "Export output..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFileSaveOutputAs() ) );
    aAction->addTo( m_pFile );

	m_pLastOpendFiles = new QPopupMenu( m_pFile );
	connect( m_pLastOpendFiles, SIGNAL( aboutToShow() ), this, SLOT( sltUpdateLastFilesMenu() ) );
	connect( m_pLastOpendFiles, SIGNAL( activated(int) ), this, SLOT( sltLastFilesMenuActivated(int) ) );
	m_pFile->insertItem( tr( "Last files" ), m_pLastOpendFiles );

#ifndef ZAURUS
	m_pFile->insertSeparator();

	aAction = createNewAction( tr( "Print..." ), QString::null, NOT_ZAURUS( CTRL+Key_P ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltFilePrint() ) );
    aAction->addTo( m_pFile );
#endif

	m_pFile->insertSeparator();

	// "settings"
    aAction = createNewAction( tr( "About..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltShowAbout() ) );
    aAction->addTo( m_pFile );

	// ++++++++++++++++++++++++++++++++++++++++++

    aAction = createNewAction( tr( "Undo" ), QString::null, NOT_ZAURUS( CTRL+Key_Z ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditUndo() ) );
    aAction->addTo( m_pEdit );

    aAction = createNewAction( tr( "Redo" ), QString::null, NOT_ZAURUS( CTRL+Key_Y ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditRedo() ) );
    aAction->addTo( m_pEdit );

    m_pEdit->insertSeparator();

	m_pCutAction = createNewAction( tr( "Cut" ), QString::null, NOT_ZAURUS( CTRL+Key_X ), this, 0 );
    connect( m_pCutAction, SIGNAL( activated() ), this, SLOT( sltEditCut() ) );
    m_pCutAction->addTo( m_pEdit );

    m_pCopyAction = createNewAction( tr( "Copy" ), QString::null, NOT_ZAURUS( CTRL+Key_C ), this, 0 );
    connect( m_pCopyAction, SIGNAL( activated() ), this, SLOT( sltEditCopy() ) );
    m_pCopyAction->addTo( m_pEdit );

    m_pPasteAction = createNewAction( tr( "Paste" ), QString::null, NOT_ZAURUS( CTRL+Key_V ), this, 0 );
    connect( m_pPasteAction, SIGNAL( activated() ), this, SLOT( sltEditPaste() ) );
    m_pPasteAction->addTo( m_pEdit );

    m_pEdit->insertSeparator();

    aAction = createNewAction( tr( "Find..." ), QPERESOURCE3( "find",getInstallationPath()+QString("inline_img/find.png"),find_ico,find_png ) QString::null, NOT_ZAURUS( CTRL+Key_F ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditFindText() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( m_pEdit );

    aAction = createNewAction( tr( "Find next" ), QPERESOURCE3( "forward",getInstallationPath()+QString("inline_img/forward.png"),forward_ico,forward_png ) QString::null, NOT_ZAURUS( Key_F3 ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditFindNext() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( m_pEdit );

    aAction = createNewAction( tr( "Replace..." ), QString::null, NOT_ZAURUS( CTRL+Key_H ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditReplaceText() ) );
    aAction->addTo( m_pEdit );

    m_pFindInFilesAction = createNewAction( tr( "Find in files..." ), QString::null, NOT_ZAURUS( CTRL+SHIFT+Key_F ), this, 0 );
    connect( m_pFindInFilesAction, SIGNAL( activated() ), this, SLOT( sltEditFindInFiles() ) );
    m_pFindInFilesAction->addTo( m_pEdit );

	aAction = createNewAction( tr( "Goto..." ), QString::null, NOT_ZAURUS( CTRL+Key_G ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltEditGotoLine() ) );
    aAction->addTo( m_pEdit );

    m_pEdit->insertSeparator();

    aAction = createNewAction( tr( "Clear output" ), QPERESOURCE3( "editdelete",getInstallationPath()+QString("inline_img/editdelete.png"),editdelete_ico,editdelete_png ) QString::null, NOT_ZAURUS( Key_F6 ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltClearOutput() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( m_pEdit );

	// ++++++++++++++++++++++++++++++++++++++++++

    aAction = createNewAction( tr( "Run" ), QPERESOURCE3(/*"exec"*/"play",getInstallationPath()+QString("inline_img/play.png"),play_ico,play_png) QString::null, NOT_ZAURUS( Key_F5 ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltRun() ) );
    aAction->addTo( m_pButtonBar );
    aAction->addTo( m_pEdit );

#ifndef ZAURUS
    aAction = createNewAction( tr( "Run act." ), QString::null, NOT_ZAURUS( SHIFT+Key_F5 ), this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltActRun() ) );
    aAction->addTo( m_pEdit );
#endif

    aAction = createNewAction( tr( "Kill" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltStopInteractiveModus() ) );
    aAction->addTo( m_pEdit );

	// ++++++++++++++++++++++++++++++++++++++++++
/*
	m_pFixedAction = createNewAction( tr( "Fixed" ), QString::null, 0, this, 0, TRUE );
    connect( m_pFixedAction, SIGNAL( activated() ), this, SLOT( sltSelectFixed() ) );
    m_pFixedAction->addTo( m_pView );
	m_pHelveticaAction = createNewAction( tr( "Helvetica" ), QString::null, 0, this, 0, TRUE );
    connect( m_pHelveticaAction, SIGNAL( activated() ), this, SLOT( sltSelectHelvetica() ) );
    m_pHelveticaAction->addTo( m_pView );
	m_pUnifontAction = createNewAction( tr( "Unifont" ), QString::null, 0, this, 0, TRUE );
    connect( m_pUnifontAction, SIGNAL( activated() ), this, SLOT( sltSelectUnifont() ) );
    m_pUnifontAction->addTo( m_pView );

    m_pView->insertSeparator();

	m_pLargeAction = createNewAction( tr( "Large" ), QString::null, 0, this, 0, TRUE );
    connect( m_pLargeAction, SIGNAL( activated() ), this, SLOT( sltSelectLarge() ) );
    m_pLargeAction->addTo( m_pView );
	m_pMediumAction = createNewAction( tr( "Medium" ), QString::null, 0, this, 0, TRUE );
    connect( m_pMediumAction, SIGNAL( activated() ), this, SLOT( sltSelectMedium() ) );
    m_pMediumAction->addTo( m_pView );
	m_pSmallAction = createNewAction( tr( "Small" ), QString::null, 0, this, 0, TRUE );
    connect( m_pSmallAction, SIGNAL( activated() ), this, SLOT( sltSelectSmall() ) );
    m_pSmallAction->addTo( m_pView );
*/
    aAction = createNewAction( tr( "Font..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltSelectFont() ) );
    aAction->addTo( m_pView );

	// "settings"
    aAction = createNewAction( tr( "Configuration..." ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltConfiguration() ) );
    aAction->addTo( m_pView );

	// WARNING: the sequence of the menu has to fit with the definitions of the language ids !!!
	m_pView->insertItem( tr("Language"), m_pLanguages );
	m_pLanguages->insertItem( tr("none"), LANG_NONE );
	m_pLanguages->insertItem( tr("minscript"), LANG_MINSCRIPT );
	m_pLanguages->insertItem( tr("C++"), LANG_CPP );
	m_pLanguages->insertItem( tr("C#"), LANG_CSHARP );
	m_pLanguages->insertItem( tr("Java"), LANG_JAVA );
	m_pLanguages->insertItem( tr("JavaScript"), LANG_JAVASCRIPT );
	m_pLanguages->insertItem( tr("Python"), LANG_PYTHON );
	m_pLanguages->insertItem( tr("Ruby"), LANG_RUBY );
	m_pLanguages->insertItem( tr("Perl"), LANG_PERL );
		m_pLanguages->insertItem( tr("PHP"), LANG_PHP );
	m_pLanguages->insertItem( tr("Lua"), LANG_LUA );
	m_pLanguages->insertItem( tr("Lisp"), LANG_LISP );
		m_pLanguages->insertItem( tr("Tcl"), LANG_TCL );
	m_pLanguages->insertItem( tr("IDL"), LANG_IDL );
	m_pLanguages->insertItem( tr("SQL"), LANG_SQL );
	m_pLanguages->insertItem( tr("HTML"), LANG_HTML );
	m_pLanguages->insertItem( tr("CSS"), LANG_CSS );
	m_pLanguages->insertItem( tr("TeX"), LANG_TEX );
	m_pLanguages->insertItem( tr("Makefile"), LANG_MAKEFILE );
	m_pLanguages->insertItem( tr("batch"), LANG_BATCH );
	m_pLanguages->insertItem( tr("bash"), LANG_BASH );

	m_pView->insertSeparator();

    aAction = createNewAction( tr( "Zoom in" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltZoomIn() ) );
    aAction->addTo( m_pView );
    aAction = createNewAction( tr( "Zoom out" ), QString::null, 0, this, 0 );
    connect( aAction, SIGNAL( activated() ), this, SLOT( sltZoomOut() ) );
    aAction->addTo( m_pView );

	m_pView->insertSeparator();

	// in dektop-versions we have a splitter to change the output size !
	m_pMaximizeScript = createNewAction( tr( "Maximize script" ), QString::null, 0, this, 0 );
	m_pMaximizeScript->setToggleAction(true);
    connect( m_pMaximizeScript, SIGNAL( activated() ), this, SLOT( sltMaximizeScript() ) );
    m_pMaximizeScript->addTo( m_pView );
    m_pMaximizeOutput = createNewAction( tr( "Maximize output" ), QString::null, 0, this, 0 );
	m_pMaximizeOutput->setToggleAction(true);
    connect( m_pMaximizeOutput, SIGNAL( activated() ), this, SLOT( sltMaximizeOutput() ) );
    m_pMaximizeOutput->addTo( m_pView );

	m_pView->insertSeparator();

	m_pToggleWordWrap = createNewAction( tr( "Word wrap" ), QString::null, 0, this, 0 );
	m_pToggleWordWrap->setToggleAction(true);
    connect( m_pToggleWordWrap, SIGNAL( activated() ), this, SLOT( sltToggleWordWrap() ) );
    m_pToggleWordWrap->addTo( m_pView );
    m_pToggleOverwrite = createNewAction( tr( "Overwrite" ), QString::null, 0, this, 0 );
	m_pToggleOverwrite->setToggleAction(true);
    connect( m_pToggleOverwrite, SIGNAL( activated() ), this, SLOT( sltToggleOverwrite() ) );
    m_pToggleOverwrite->addTo( m_pView );
    m_pCodeFolding = createNewAction( tr( "Code folding" ), QString::null, 0, this, 0 );
	m_pCodeFolding->setToggleAction(true);
    connect( m_pCodeFolding, SIGNAL( activated() ), this, SLOT( sltCodeFolding() ) );
    m_pCodeFolding->addTo( m_pView );
    m_pMarkAsMain = createNewAction( tr( "Mark as main" ), QString::null, 0, this, 0 );
    connect( m_pMarkAsMain, SIGNAL( activated() ), this, SLOT( sltViewMarkActAsMain() ) );
    m_pMarkAsMain->addTo( m_pView );

	m_pView->insertSeparator();
	// in Eclipse: CTRL+'/'
    m_pCommentAction = createNewAction( tr( "Comment block" ), QString::null, NOT_ZAURUS( CTRL+Key_M ), this, 0 );
    connect( m_pCommentAction, SIGNAL( activated() ), this, SLOT( sltCommentSelection() ) );
    m_pCommentAction->addTo( m_pView );

	// in Eclipse: CTRL+'\'
    m_pUnCommentAction = createNewAction( tr( "Uncomment block" ), QString::null, NOT_ZAURUS( CTRL+Key_U ), this, 0 );
    connect( m_pUnCommentAction, SIGNAL( activated() ), this, SLOT( sltUnCommentSelection() ) );
    m_pUnCommentAction->addTo( m_pView );

//#ifdef ZAURUS
//	m_pGotoBegin = createNewAction( tr( "Begin" ), QString::null, CTRL+Key_B, this, 0, TRUE );
//	connect( m_pGotoBegin, SIGNAL( activated() ), this, SLOT( sltGotoBegin() ) );
//	m_pGotoEnd = createNewAction( tr( "End" ), QString::null, CTRL+Key_E, this, 0, TRUE );
//	connect( m_pGotoEnd, SIGNAL( activated() ), this, SLOT( sltGotoEnd() ) );
//#endif

	// *************************************************************
	// *************************************************************

	m_pStack = new QWidgetStack( this );
    setCentralWidget( m_pStack );

    m_pTabWidget = new QTabWidget( m_pStack, "pTabWidget" );

	m_pStack->addWidget( m_pTabWidget, get_unique_id() );

#ifndef ZAURUS
	if( m_bAutoFileLoad && m_aLastOpendFilesList.count()>0 )
	{
		bool bAnyFileOpend = false;
		for( unsigned int i=0; i<m_aLastOpendFilesList.count(); i++ )
		{
			QPair<int,int> * pCursorPos = m_aLastCursorPos.at(i);
			int x = pCursorPos->first;
			int y = pCursorPos->second;
			bool bImportOk = DoImportFile( *(m_aLastOpendFilesList.at(i)), x, y );
			bAnyFileOpend = bAnyFileOpend || bImportOk;
		}
		if( !bAnyFileOpend )
		{
			AddDocument( g_iDocCount++ );
		}
	}
	else
	{
		AddDocument( g_iDocCount++ );
	}

	m_pTabWidget->setCurrentPage( m_iLastActFileIndex );

	if( m_iLastMainScriptIndex>=0 && GetScriptFor( m_iLastMainScriptIndex ) )
	{
		GetScriptFor( m_iLastMainScriptIndex )->setMainFile(true);
		// und alles aktualisieren
		ForAllDocumentScripts( &MyQCodeEdit::sltUpdate );
	}

	if( m_iShowStartupFileDialog )
	{
	    sltFileOpen();
	}
	else
	{
		sltShowScript();
		updateCaption();
	}

    resize( m_iLastPosDX, m_iLastPosDY );
	move( m_iLastPosX, m_iLastPosY );
#endif
	m_pProcess = new QProcess( this );
	m_pProcess->setCommunication( QProcess::Stdout|QProcess::Stderr|QProcess::Stdin|QProcess::DupStderr );

	connect( m_pProcess, SIGNAL(readyReadStdout()), this, SLOT(sltReadFromStdout()) );
    connect( m_pProcess, SIGNAL(readyReadStderr()), this, SLOT(sltReadFromStderr()) );
    connect( m_pProcess, SIGNAL(processExited()), this, SLOT(sltProcessExited()) );

	sltStartInteractiveModus( m_bInteractiveModus );

#ifndef _with_opie_filedialog
#ifdef ZAURUS
	m_pExportFileDlg = new FileDialogS(this, "save",TRUE, WType_Modal, TRUE);
	connect( m_pExportFileDlg, SIGNAL( doneLoad() ), this, SLOT( sltDoImport() ) );
    connect( m_pExportFileDlg, SIGNAL( done() ), this, SLOT( sltDoExport() ) );
	m_pExportFileDlg->adjustSize();
#endif
#endif
}

/*
 *  Destroys the object and frees any allocated resources
 */
VisiScript::~VisiScript()
{
#ifndef _with_opie_filedialog
#ifdef ZAURUS
	delete m_pExportFileDlg;
#endif
#endif
	if( m_pFindDlg )
	{
		delete m_pFindDlg;
		m_pFindDlg = 0;
	}
	if( m_pReplaceDlg )
	{
		delete m_pReplaceDlg;
		m_pReplaceDlg = 0;
	}
	if( m_pFindInFilesDlg )
	{
		delete m_pFindInFilesDlg;
		m_pFindInFilesDlg = 0;
	}

	if( m_pAutoCompletionAPI )
	{
		delete m_pAutoCompletionAPI;
		m_pAutoCompletionAPI = 0;
	}
	if( m_pProcess ) 
	{
		delete m_pProcess;
		m_pProcess = 0;
	}
	// no need to delete child widgets, Qt does it all for us
}

void VisiScript::sltFileNew()
{
	AddDocument( g_iDocCount++ );
}

void VisiScript::updateFileSelector()
{
}

static QString GetDirectoryFromFilePath( const QString & sFileName )
{
	QString sDrive, sDir, sName, sExt;
	if( SplitPath( (const char *)sFileName, sDrive, sDir, sName, sExt ) )
	{
		return sDrive+sDir;
	}
	return "";
}

void VisiScript::sltFileOpen()
{
#ifndef ZAURUS
	QString sFileName = QFileDialog::getOpenFileName( /*"."*/m_sLastDirectory, QString::null, this );

	if( !sFileName.isEmpty() )
	{
		m_sLastDirectory = GetDirectoryFromFilePath( sFileName );

		DoImportFile( sFileName );
	}
#else
    sltFileImport();
#endif
}

void VisiScript::sltFileSave()
{
	DoExportFile( GetActScript()->fileName(), /*bSaveOutput=*/false );
}

void VisiScript::sltFileSaveAs()
{
#ifndef ZAURUS
	QString sFileName = QFileDialog::getSaveFileName( /*"."*/m_sLastDirectory, QString::null, this );

	if( !sFileName.isEmpty() )
	{
		GetActScript()->setFileName( sFileName );

		if( DoExportFile( GetActScript()->fileName(), /*bSaveOutput=*/false ) )
		{
			m_sLastDirectory = GetDirectoryFromFilePath( sFileName );
			PushLastFileName( sFileName );
		}
	}
#else
    sltFileExport();
#endif
}

void VisiScript::sltFileImport()
{
#if defined(_with_zfax_filedialog) && !defined(ZAURUS)
	fileBrowser dlg(this, true);

	if( dlg.exec() && !dlg.selectedFileName.isEmpty() )
	{
		DoImportFile( dlg.selectedFileName );
	}
#else
#ifdef _with_opie_filedialog
	// NOT USED JET !!!

	//CheckData();

	QString sFileName = OFileDialog::getOpenFileName( OFileSelector::NORMAL );

	DoImportFile( sFileName );
#else
	m_pExportFileDlg->setLoad( TRUE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
	m_pExportFileDlg->adjustSize();
#endif
#endif
}

void VisiScript::sltFileExport()
{
#if defined(_with_zfax_filedialog) && !defined(ZAURUS)
	fileBrowser dlg(this, false);

	if( dlg.exec() && !dlg.selectedFileName.isEmpty() )
	{
		DoExportFile( dlg.selectedFileName, /*bSaveOutput=*/false );
	}
#else
#ifdef _with_opie_filedialog
	// NOT USED JET !!!
	
	QString sFileName = OFileDialog::getSaveFileName( OFileSelector::NORMAL );
	//cout << "Export: " << (const char *)sFileName << endl;

	if( !sFileName.isNull() )
	{
		DoExportFile( sFileName, /*bSaveOutput=*/false );
	}
#else
	//m_bSaveOutput = false;
	m_pExportFileDlg->setLoad( FALSE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
	m_pExportFileDlg->adjustSize();
#endif
#endif
}

void VisiScript::sltFileSaveOutputAs()
{
#if defined(_with_zfax_filedialog) && !defined(ZAURUS)
	fileBrowser dlg(this, false);

	if( dlg.exec() && !dlg.selectedFileName.isEmpty() )
	{
		QString sText = GetActOutput()->text();
		/*bool bOk =*/ WriteTextFile( dlg.selectedFileName/*.latin1()*/, sText, /*append=*/false, m_bSetUtf8 );
	}
#else
#ifdef _with_opie_filedialog
	// NOT USED JET !!!

	QString sFileName = OFileDialog::getSaveFileName( OFileSelector::NORMAL );
	//cout << "SaveOutput: " << (const char *)sFileName << endl;

	if( !sFileName.isNull() )
	{
		QString sText = GetActOutput()->text();
		/*bool bOk =*/ WriteTextFile( sFileName/*.latin1()*/, sText, /*append=*/false, m_bSetUtf8 );
	}
#else
	//m_bSaveOutput = true;
	m_pExportFileDlg->setLoad( FALSE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
	m_pExportFileDlg->adjustSize();
#endif
#endif
}

void VisiScript::sltFilePrint()
{
#ifndef ZAURUS
	QextScintillaPrinter aPrinter;
	
	aPrinter.setFullPage( true );
	if( aPrinter.setup( this ) )
	{
//		if( GetActScript()->hasFocus() )
//		{
			aPrinter.printRange( GetActScript() );
//		}
//		else
//		{
// TODO --> print the output window... 
			//aPrinter.printRange( GetActOutput() );
//		}
	}
#endif
}

#ifndef _with_opie_filedialog
void VisiScript::sltDoImport()
{
#ifdef ZAURUS
	//CheckData();

	QString * pFileName = m_pExportFileDlg->getAbsSelectedFileName();

	m_sLastDirectory = m_pExportFileDlg->getDirectory();

	DoImportFile( *pFileName );

	delete pFileName;
#endif
}

void VisiScript::sltDoExport()
{
#ifdef ZAURUS
	QString * pFileName = m_pExportFileDlg->getAbsSelectedFileName();

	m_sLastDirectory = m_pExportFileDlg->getDirectory();

	DoExportFile( *pFileName, /*bSaveOutput=*/false );

	delete pFileName;
#endif
}
#endif

static void SetComboBoxToText( QComboBox * pCombo, const QString & sText )
{
	for( int i=0; i<pCombo->count(); i++ )
	{
		if( pCombo->text( i ) == sText )
		{
			pCombo->setCurrentItem( i );
			break;
		}
	}
}

void VisiScript::updateExternalProgs( QComboBox * pCombo )
{
	pCombo->setDuplicatesEnabled( false );
	pCombo->clear();

	for( unsigned int i=0; i<m_aExternalProgsList.count(); i++ )
	{
		QString sExternalProg = *(m_aExternalProgsList.at( m_aExternalProgsList.count()-i-1 ));

		pCombo->insertItem( sExternalProg, i );
	}
}

void VisiScript::sltConfiguration()
{
	ConfigureDlgImpl * pDlg = new ConfigureDlgImpl( this, "configuredlg", /*bModal*/TRUE );

	pDlg->adjustSize();

	pDlg->m_pScriptCmd->setText( m_sScriptCmd );

	updateExternalProgs( pDlg->m_pScriptCmdList );

	SetComboBoxToText( pDlg->m_pScriptCmdList, m_sScriptCmd );

	// update the combo box for the last script arguments strings
	for( unsigned int i=0; i<m_aLastScriptArgumentsList.count(); i++ )
	{
		pDlg->m_pScriptArguments->insertItem( *(m_aLastScriptArgumentsList.at( i )), 0 );
	}

	SetComboBoxToText( pDlg->m_pScriptArguments, m_sScriptArguments );

	pDlg->m_pTabSpace->setText( QString::number( m_iTabSpace ) );
	pDlg->m_pSpacesForTabs->setChecked( (bool)m_iSpacesForTabs );
	pDlg->m_pShowWhitespaces->setChecked( (bool)m_iShowWhitespaces );
	pDlg->m_pShowLineNumbers->setChecked( (bool)m_iShowLineNumbers );
	pDlg->m_pAutoIndent->setChecked( (bool)m_iAutoIndent );
	pDlg->m_pCodeFolding->setChecked( (bool)m_iCodeFolding );
	pDlg->m_pOutputRatio->setValue( m_iOutputRatio );
	pDlg->m_pInteractionModus->setChecked( (bool)m_bInteractiveModus );
	pDlg->m_pChangeActDir->setChecked( (bool)m_bChangeActDir );
	pDlg->m_pAutoSave->setChecked( (bool)m_bAutoSave );
	pDlg->m_pBraceMatching->setChecked( (bool)m_bBraceMatching );
	pDlg->m_pAutoCompleteThresh->setText( QString::number( m_iAutoCompletionThresh ) );
	pDlg->m_pAutoClearOutput->setChecked( (bool)m_bAutoClearOutput );
	pDlg->m_pFlagUtf8->setChecked( (bool)m_bSetUtf8 );

	pDlg->m_pAutoCompletion->setText( m_sAutoCompletionFileName );

	pDlg->m_pExecuteCmd->setText( m_sExeclCmd );

	if( m_iExecuteMode == EXECUTE_MODE_POPEN )
	{
		pDlg->m_pUsepopen->setChecked( TRUE );
	}
	if( m_iExecuteMode == EXECUTE_MODE_EXECL )
	{
		pDlg->m_pUseexecl->setChecked( TRUE );
	}
	if( m_iExecuteMode == EXECUTE_MODE_QPROCESS )
	{
		pDlg->m_pUseqprocess->setChecked( TRUE );
	}
	if( m_iShowStartupFileDialog==0 )
	{
		pDlg->m_pShowFileDialog->setChecked( FALSE );
	}
	else
	{
		pDlg->m_pShowFileDialog->setChecked( TRUE );
	}
	if( m_bAutoFileLoad )
	{
		pDlg->m_pAutoFileLoad->setChecked( TRUE );
	}
	else
	{
		pDlg->m_pAutoFileLoad->setChecked( FALSE );
	}

	if( m_iEolMode==QextScintilla::EolWindows )
	{
		pDlg->m_pCRLFModus->setCurrentItem( 0 );
	}
	else if( m_iEolMode==QextScintilla::EolUnix )
	{
		pDlg->m_pCRLFModus->setCurrentItem( 1 );
	}
	else if( m_iEolMode==QextScintilla::EolMac )
	{
		pDlg->m_pCRLFModus->setCurrentItem( 2 );
	}

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		// ok, pressed
		m_sScriptCmd = pDlg->m_pScriptCmd->text();

		PushToList( m_sScriptCmd, m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );

		m_sScriptArguments = pDlg->m_pScriptArguments->currentText();
		PushToList( m_sScriptArguments, m_aLastScriptArgumentsList, MAX_LAST_SEARCH_STRINGS );

		m_sAutoCompletionFileName = pDlg->m_pAutoCompletion->text();

		m_iSpacesForTabs = (int)pDlg->m_pSpacesForTabs->isChecked();
		m_iShowWhitespaces = (int)pDlg->m_pShowWhitespaces->isChecked();
		m_iShowLineNumbers = (int)pDlg->m_pShowLineNumbers->isChecked();
		m_iAutoIndent = (int)pDlg->m_pAutoIndent->isChecked();
		m_iCodeFolding = (int)pDlg->m_pCodeFolding->isChecked();
		m_iOutputRatio = pDlg->m_pOutputRatio->value();
		m_bInteractiveModus = pDlg->m_pInteractionModus->isChecked();
		m_bChangeActDir = pDlg->m_pChangeActDir->isChecked();
		m_bAutoSave = pDlg->m_pAutoSave->isChecked();
		m_bBraceMatching = pDlg->m_pBraceMatching->isChecked();
		m_bAutoClearOutput = pDlg->m_pAutoClearOutput->isChecked();
		m_bSetUtf8 = pDlg->m_pFlagUtf8->isChecked();
		m_bAutoFileLoad = pDlg->m_pAutoFileLoad->isChecked();
		m_iAutoCompletionThresh = pDlg->m_pAutoCompleteThresh->text().toInt();
		int iMode = pDlg->m_pCRLFModus->currentItem();
		if( iMode == 0 )
		{
			m_iEolMode = QextScintilla::EolWindows;
		}
		else if( iMode == 1 )
		{
			m_iEolMode = QextScintilla::EolUnix;
		}
		else if( iMode == 2 )
		{
			m_iEolMode = QextScintilla::EolMac;
		}

		m_iTabSpace = pDlg->m_pTabSpace->text().toInt();
#ifdef ZAURUS
		QMultiLineEdit::setDefaultTabStop( m_iTabSpace );
#else
		ForAllDocumentOutput( m_iTabSpace, (void (MyQMultiLineEdit::*)(int))&MyQMultiLineEdit::setTabStopWidth );
#endif
		ForAllDocumentScripts( m_iTabSpace, &MyQCodeEdit::setTabWidth );
		ForAllDocumentScripts( true, (void (MyQCodeEdit::*)(bool))&MyQCodeEdit::repaint );

		m_sExeclCmd = pDlg->m_pExecuteCmd->text();

		if( pDlg->m_pUsepopen->isChecked() )
		{
			m_iExecuteMode = EXECUTE_MODE_POPEN;
			// interactive modus only for QProcess Mode possible !
			m_bInteractiveModus = false;			// Seiteneffekt !!!
		}
		if( pDlg->m_pUseexecl->isChecked() )
		{
			m_iExecuteMode = EXECUTE_MODE_EXECL;
			// interactive modus only for QProcess Mode possible !
			m_bInteractiveModus = false;			// Seiteneffekt !!!
		}
		if( pDlg->m_pUseqprocess->isChecked() )
		{
			m_iExecuteMode = EXECUTE_MODE_QPROCESS;
		}
		if( pDlg->m_pShowFileDialog->isChecked() )
		{
			m_iShowStartupFileDialog = 1;
		}
		else
		{
			m_iShowStartupFileDialog = 0;
		}
		
		UpdateFlags();
		UpdateScriptOutputRatio();

		sltStopInteractiveModus();
		sltStartInteractiveModus( m_bInteractiveModus );

		saveInit();
	}
	delete pDlg;
}

void VisiScript::sltShowAbout()
{
	AboutDlg * pDlg = new AboutDlg( this, "aboutdlg", /*bModal*/TRUE );

	pDlg->adjustSize();

	/*int iRet =*/ pDlg->exec();

	delete pDlg;
}

void VisiScript::sltUpdateLastFilesMenu()
{
	m_pLastOpendFiles->clear();

	for( unsigned int i=0; i<m_aLastFilesList.count(); i++ )
	{
		QString sLastFileName = *(m_aLastFilesList.at( m_aLastFilesList.count()-i-1 ));

		// ** show filename with number 

#ifndef ZAURUS
		sLastFileName = QString( "&" ) + QString::number( i ) + QString( " " ) + 
						sLastFileName;
#endif

		m_pLastOpendFiles->insertItem( sLastFileName, i );
	}
}

void VisiScript::sltLastFilesMenuActivated( int iIndex )
{
	QString sFileName = m_pLastOpendFiles->text( iIndex );

#ifndef ZAURUS
	sFileName = sFileName.right( sFileName.length()-3 );
#endif

	DoImportFile( sFileName );
}

void VisiScript::sltViewMarkActAsMain()
{
    if( GetActScript()->isMainFile() )
    {
    	// das aktuelle Skript als main zuruecksetzen
    	GetActScript()->setMainFile(false);
    }
    else
    {
    	// erst alle Skripte als nicht main setzen
    	ForAllDocumentScripts( false, &MyQCodeEdit::setMainFile );
    
    	// das aktuelle Skript als main markieren
    	GetActScript()->setMainFile(true);
    }
	// und alles aktualisieren
	ForAllDocumentScripts( &MyQCodeEdit::sltUpdate );
}

void VisiScript::sltUpdateChilds()
{
	UpdateFont( true );
}

#ifdef ZAURUS
static int min( int i1, int i2 )
{
	return i1<i2 ? i1 : i2;
}
#endif

static int convert_to_max( int i )
{
	if( i<0 )
	{
		return (1 << 16)-1;
	}
	return i;
}
static int find_min( int i1, int i2, int i3, int i4, int i5 )
{
	return min(convert_to_max(i1),min(convert_to_max(i2),min(convert_to_max(i3),min(convert_to_max(i4),convert_to_max(i5)))));;
}
static int find_separator_char( const QString & strLine, int iFound, bool bIgnorePathChars=false )
{
	// try some line-number stop characters
	int i1 = strLine.find(",",iFound);
	int i2 = strLine.find(";",iFound);
	int i3 = bIgnorePathChars ? -1 : strLine.find(".",iFound);
	int i4 = bIgnorePathChars ? -1 : strLine.find(":",iFound);
	int i5 = bIgnorePathChars ? -1 : strLine.find(" ",iFound);
	int i = find_min(i1,i2,i3,i4,i5);
	return i;
}

void VisiScript::sltOutputDoubleClicked( int iPara, int /*iPos*/ )
{
	// search for the line tag in the output window and try to find a line-number
	// example for Python:   File "_tsf.xyz", line 3, in ?
	// example for ruby: Error: _tsf.xyz:2: undefined local variable or method `x' for #<Object:0x949618> (NameError)
	// example for perl: Error: Scalar found where operator expected at _tsf.xyz line 3, near "$x" ...
	const QString & strLine = GetActOutput()->textLine(iPara);
	int iFoundFile = strLine.find(c_strFileTag,0,/*case_sensitive*/false);
	if( iFoundFile>=0 )
	{
		iFoundFile = iFoundFile + c_strFileTag.length();
	}
	else
	{
		// try it with the english text...
		iFoundFile = strLine.find(c_strFileTagOrg,0,/*case_sensitive*/false);
		if( iFoundFile>=0 )
		{
			iFoundFile = iFoundFile + c_strFileTagOrg.length();
		}
	}
	if( iFoundFile>=0 )
	{
		int i = find_separator_char(strLine,iFoundFile,true);
		if( i>=0 )
		{
			int iIndex = 0;
			QString strNewFile = stripPath( strLine.mid(iFoundFile,i-iFoundFile) );
			// is the script allready loaded ?
			MyQCodeEdit * pScript = GetScriptForFileName(strNewFile,&iIndex);
			if( pScript==0 )
			{
				// no, load it !
				DoImportFile( strNewFile );
			}
			else
			{
				// yes, activate the allready loaded script to update the cursor position
				m_pTabWidget->setCurrentPage(iIndex);
			}
		}
	}
	int iFound = strLine.find(c_strLineTag,0,/*case_sensitive*/false);
	if( iFound>=0 )
	{
		iFound = iFound + c_strLineTag.length();
	}
	else
	{
		// try it with the english text...
		iFound = strLine.find(c_strLineTagOrg,0,/*case_sensitive*/false);
		if( iFound>=0 )
		{
			iFound = iFound + c_strLineTagOrg.length();
		}
	}
	if( iFound>=0 )
	{
		int i = find_separator_char(strLine,iFound);
		if( i>=0 )
		{
			int iFoundPos = strLine.mid(iFound,i-iFound).toInt();
			int iLineNo = iFoundPos-1;
			MyQCodeEdit * pScript = GetActScript();
			pScript->setFocus();
			pScript->setCursorPosition( iLineNo, 0 );
			// damit die selektierte Zeile niemals am Ende des Fensters steht !
			pScript->ensureLineVisible( iLineNo+5 );
			// select the complete line...
			pScript->setSelection( iLineNo, 0, iLineNo, pScript->text(iLineNo).length() );
		}
	}
}

void VisiScript::sltPositionChanged( int iLine, int iPos )
{
#ifndef ZAURUS
	QString s;

	s.sprintf( tr("line=%d pos=%d"), iLine+1, iPos+1 );

	m_pStatusBarMsg->setText( s );
#endif
	if( m_iActLine != iLine && GetActScript()->isLastKeyReturn() && m_bInteractiveModus )
	{
		QString sLastLine = GetActScript()->text( m_iActLine );

		// start process if not running
		sltStartInteractiveModus( m_bInteractiveModus );
		// write last enterd line
		sltWriteToStdin( sLastLine );
	}
	m_iActLine = iLine;
	m_iActPos = iPos;
}

void VisiScript::sltReadFromStdout()
{
	while( m_pProcess->canReadLineStdout() )
	{
		QString sOutput = m_pProcess->readLineStdout();
		sltAddLineToOutput( sOutput );
	}
}

void VisiScript::sltReadFromStderr()
{
	while( m_pProcess->canReadLineStderr() )
	{
		QString sOutput = m_pProcess->readLineStderr();
		sltAddLineToOutput( sOutput );
	}
}

void VisiScript::sltProcessExited()
{
}

void VisiScript::sltWriteToStdin( const QString & sText )
{
	m_pProcess->writeToStdin( sText+"\n" );
}

void VisiScript::sltStartInteractiveModus( bool bEnable )
{
	if( bEnable && m_pProcess && !m_pProcess->isRunning() )
	{
		m_pProcess->clearArguments();

		QStringList aList = QStringList::split( " ", m_sScriptCmd );
		for( unsigned int i=0; i<aList.count(); i++ )
		{
			QString sTemp = aList[i];
			m_pProcess->addArgument( sTemp );
		}

		bool bOk = m_pProcess->start();

		if( !bOk )
		{
			QString sMsg;
			sMsg.sprintf( tr( "Error: Process %s could not be started!" ), (const char *)m_sScriptCmd );
			sltAddLineToOutput( sMsg );
			//QMessageBox::information( this, tr( "Start Process" ), sMsg, QMessageBox::Ok );
		}
	}
}

void VisiScript::sltStopInteractiveModus()
{
	if( m_pProcess && m_pProcess->isRunning() )
	{
		m_pProcess->kill();

		QString sMsg;
		sMsg.sprintf( tr( "Process %s killed!" ), (const char *)m_sScriptCmd );
		sltAddLineToOutput( sMsg );
	}
}

void VisiScript::sltSelectFont()
{
#ifndef QT_NO_FONTDIALOG
	bool bOk = false;
	QFont aFont( m_sFontName, m_iFontSize );

	aFont = QFontDialog::getFont( &bOk, aFont, this );
	if( bOk )
	{
		m_sFontName = aFont.family();
		m_sFontStyle = aFont.styleHint();
		m_iFontSize = aFont.pointSize();
		UpdateFont( true );
		saveInit();
	}
#else
	FontDlg * pDlg = new FontDlg( this, "fontdlg", /*bModal*/TRUE );

	pDlg->m_pFontWidget->setSelectedFont( m_sFontName, m_sFontStyle, m_iFontSize );

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_sFontName = pDlg->m_pFontWidget->fontFamily();
		m_sFontStyle = pDlg->m_pFontWidget->fontStyle();
		m_iFontSize = pDlg->m_pFontWidget->fontSize();
		UpdateFont( true );
		saveInit();
	}

	delete pDlg;
#endif
}
/*
void VisiScript::sltSelectHelvetica()
{
	m_pFixedAction->setOn( false );
	m_pUnifontAction->setOn( false );
	m_sFontName = "Helvetica";
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectFixed()
{
	m_pHelveticaAction->setOn( false );
	m_pUnifontAction->setOn( false );
	m_sFontName = "fixed";
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectUnifont()
{
	m_pFixedAction->setOn( false );
	m_pHelveticaAction->setOn( false );
	m_sFontName = "Unifont";
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectSmall()
{
	m_pMediumAction->setOn( false );
	m_pLargeAction->setOn( false );
	m_iFontSize = c_iSmall;
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectMedium()
{
	m_pSmallAction->setOn( false );
	m_pLargeAction->setOn( false );
	m_iFontSize = c_iMedium;
	UpdateFont();
	saveInit();
}

void VisiScript::sltSelectLarge()
{
	m_pMediumAction->setOn( false );
	m_pSmallAction->setOn( false );
	m_iFontSize = c_iLarge;
	UpdateFont();
	saveInit();
}
*/

void VisiScript::sltZoomIn()
{
	m_iFontSize++;
	UpdateFont( true );
	saveInit();
}

void VisiScript::sltZoomOut()
{
	m_iFontSize--;
	UpdateFont( true );
	saveInit();
}

void VisiScript::sltMaximizeScript()
{
	if( !m_pMaximizeScript->isOn() )
	{
		SetOutputSize( m_iOutputRatio );
	}
	else
	{
		m_pMaximizeOutput->setOn( false );
		SetOutputSize( c_iMinOutputRatio );
	}
}

void VisiScript::sltMaximizeOutput()
{
	if( !m_pMaximizeOutput->isOn() )
	{
		SetOutputSize( m_iOutputRatio );
	}
	else
	{
		m_pMaximizeScript->setOn( false );
		SetOutputSize( c_iMaxOutputRatio );
	}
}

void VisiScript::sltToggleWordWrap()
{
	if( !m_pToggleWordWrap->isOn() )
	{
		ForAllDocumentScripts( false, &MyQCodeEdit::setWordWrap );
		m_iWordWrap = 0;
	}
	else
	{
		ForAllDocumentScripts( true, &MyQCodeEdit::setWordWrap );
		m_iWordWrap = 1;
	}
}

void VisiScript::sltToggleOverwrite()
{
	if( !m_pToggleOverwrite->isOn() )
	{
		ForAllDocumentScripts( false, &MyQCodeEdit::setOverwriteMode );
		m_iOverwrite = 0;
	}
	else
	{
		ForAllDocumentScripts( true, &MyQCodeEdit::setOverwriteMode );
		m_iOverwrite = 1;
	}
}

void VisiScript::sltCodeFolding()
{
	GetActScript()->foldAll();
}

void VisiScript::UpdateFont( MyQCodeEdit * pScript, MyQMultiLineEdit * pOutput )
{
	QFontDatabase aFontDB;
	int iWeight = aFontDB.weight( m_sFontName, m_sFontStyle );
	bool bItalic = aFontDB.italic( m_sFontName, m_sFontStyle );
	QFont aTempFont( m_sFontName, m_iFontSize, iWeight, bItalic );

	UpdateFont( pScript, pOutput, aTempFont );
}

void VisiScript::UpdateFont( MyQCodeEdit * pScript, MyQMultiLineEdit * pOutput, const QFont & aFont )
{
	if( pScript )
	{
		pScript->setFont( aFont );
		pScript->updateFontForLexer( aFont );
	}
	if( pOutput )
	{
		pOutput->setFont( aFont );
	}
}

void VisiScript::UpdateFont( bool bForAllDocuments )
{
	QFontDatabase aFontDB;
	int iWeight = aFontDB.weight( m_sFontName, m_sFontStyle );
	bool bItalic = aFontDB.italic( m_sFontName, m_sFontStyle );
	QFont aTempFont( m_sFontName, m_iFontSize, iWeight, bItalic );

	if( bForAllDocuments )
	{
		for( int i=0; i<GetDocumentCount(); i++ )
		{
			UpdateFont( GetScriptFor( i ), GetOutputFor( i ), aTempFont );
		}
	}
	else
	{
		UpdateFont( GetActScript(), GetActOutput(), aTempFont );
	}
}

void VisiScript::SetAutoCompletionSource( QextScintilla::AutoCompletionSource aSrc, int iThresh, QextScintillaAPIs * pAPIs )
{
	ForAllDocumentScripts( aSrc, &MyQCodeEdit::setAutoCompletionSource );
	ForAllDocumentScripts( iThresh, &MyQCodeEdit::setAutoCompletionThreshold );
	// BUG BUG BUG (folgende Zeile beim ersten Compilieren unter Windows auskommentieren...???)
	/**/ForAllDocumentScripts( pAPIs, &MyQCodeEdit::setAutoCompletionAPIs );
	ForAllDocumentScripts( pAPIs, &MyQCodeEdit::setCallTipsAPIs );
}

void VisiScript::UpdateFlags()
{
	// restore values form the init/configuration file
	if( m_iWordWrap==0 )
	{
		m_pToggleWordWrap->setOn( false );
		ForAllDocumentScripts( false, &MyQCodeEdit::setWordWrap );
	}
	else
	{
		m_pToggleWordWrap->setOn( true );
		ForAllDocumentScripts( true, &MyQCodeEdit::setWordWrap );
	}
	if( m_iOverwrite==0 /*GetActScript() && !GetActScript()->overwriteMode()*/ )
	{
		m_pToggleOverwrite->setOn( false );
		ForAllDocumentScripts( false, &MyQCodeEdit::setOverwriteMode );
	}
	else
	{
		m_pToggleOverwrite->setOn( true );
		ForAllDocumentScripts( true, &MyQCodeEdit::setOverwriteMode );
	}
	if( m_iSpacesForTabs==0 )
	{
		ForAllDocumentScripts( true, &MyQCodeEdit::setIndentationsUseTabs );
	}
	else
	{
		ForAllDocumentScripts( false, &MyQCodeEdit::setIndentationsUseTabs );
	}
	if( m_iShowWhitespaces==0 )
	{
		ForAllDocumentScripts( QextScintilla::WsInvisible, &MyQCodeEdit::setWhitespaceVisibility );
		ForAllDocumentScripts( false, &MyQCodeEdit::setEolVisibility );
	}
	else
	{
		ForAllDocumentScripts( QextScintilla::WsVisible, &MyQCodeEdit::setWhitespaceVisibility );
		ForAllDocumentScripts( true, &MyQCodeEdit::setEolVisibility );
	}
	if( m_iShowLineNumbers==0 )
	{
		ForAllDocumentScripts( 1, true, &MyQCodeEdit::setMarginLineNumbers );
		ForAllDocumentScripts( 1, 0, (void (MyQCodeEdit::*)(int,int))&MyQCodeEdit::setMarginWidth );
	}
	else
	{
		ForAllDocumentScripts( 1, true, &MyQCodeEdit::setMarginLineNumbers );
#ifdef ZAURUS
		ForAllDocumentScripts( 1, QString("999"), (void (MyQCodeEdit::*)(int,const QString &))&MyQCodeEdit::setMarginWidth );
#else
		ForAllDocumentScripts( 1, QString("9999"), (void (MyQCodeEdit::*)(int,const QString &))&MyQCodeEdit::setMarginWidth );
#endif
	}
	if( m_iAutoIndent==0 )
	{
		ForAllDocumentScripts( false, &MyQCodeEdit::setAutoIndent );
	}
	else
	{
		ForAllDocumentScripts( true, &MyQCodeEdit::setAutoIndent );
	}
	if( m_iCodeFolding==0 )
	{
		ForAllDocumentScripts( QextScintilla::NoFoldStyle, &MyQCodeEdit::setFolding );
	}
	else
	{
		ForAllDocumentScripts( QextScintilla::BoxedTreeFoldStyle, &MyQCodeEdit::setFolding );
	}
	if( m_bBraceMatching )
	{
		ForAllDocumentScripts( QextScintilla::SloppyBraceMatch, &MyQCodeEdit::setBraceMatching );
	}
	else
	{
		ForAllDocumentScripts( QextScintilla::NoBraceMatch, &MyQCodeEdit::setBraceMatching );
	}
	if( !m_sAutoCompletionFileName.isEmpty() )
	{
		if( m_sAutoCompletionFileName == "none" || m_sAutoCompletionFileName == "0" || m_sAutoCompletionFileName == "-1" )
		{
			if( m_pAutoCompletionAPI )
			{
				delete m_pAutoCompletionAPI;
				m_pAutoCompletionAPI = 0;
			}
			SetAutoCompletionSource( QextScintilla::AcsDocument, -1, m_pAutoCompletionAPI );
		}
		else
		{
			if( !m_pAutoCompletionAPI )
			{
				m_pAutoCompletionAPI = new QextScintillaAPIs();
			}
			m_pAutoCompletionAPI->clear();
			m_pAutoCompletionAPI->load( m_sAutoCompletionFileName );

			SetAutoCompletionSource( QextScintilla::AcsAPIs, m_iAutoCompletionThresh, m_pAutoCompletionAPI );
		}
	}
	else
	{
		if( m_pAutoCompletionAPI )
		{
			delete m_pAutoCompletionAPI;
			m_pAutoCompletionAPI = 0;
		}
		SetAutoCompletionSource( QextScintilla::AcsDocument, m_iAutoCompletionThresh, m_pAutoCompletionAPI );
	}
	ForAllDocumentScripts( (QextScintilla::EolMode)m_iEolMode, &MyQCodeEdit::setEolMode );
	// WARNING: im "Windows" Modus werden die Dateien 'geaendert' !
 	ForAllDocumentScripts( (QextScintilla::EolMode)m_iEolMode, &MyQCodeEdit::convertEols );
	ForAllDocumentScripts( m_bSetUtf8, &MyQCodeEdit::setUtf8 );
}

void VisiScript::UpdateLanguage( MyQCodeEdit * pScript, int iActLanguageID )
{
//	pScript->setLexer( 0, LANG_NONE );

	QextScintillaLexer * pLexer = 0;

	switch( iActLanguageID )
	{
		case LANG_NONE:
			pLexer = 0;
			break;
		case LANG_MINSCRIPT:
		case LANG_CPP:
			pLexer = new QextScintillaLexerCPP( pScript );
			break;
		case LANG_CSHARP:
			pLexer = new QextScintillaLexerCSharp( pScript );
			break;
		case LANG_PYTHON:
			pLexer = new QextScintillaLexerPython( pScript );
			break;
		case LANG_JAVA:
			pLexer = new QextScintillaLexerJava( pScript );
			break;
		case LANG_JAVASCRIPT:
			pLexer = new QextScintillaLexerJavaScript( pScript );
			break;
		case LANG_RUBY:
			pLexer = new QextScintillaLexerRuby( pScript );
			break;
		case LANG_PHP:
// TODO vorlaeufig...
			pLexer = new QextScintillaLexerCPP( pScript );
			break;
		case LANG_PERL:
			pLexer = new QextScintillaLexerPerl( pScript );
			break;
		case LANG_IDL:
			pLexer = new QextScintillaLexerIDL( pScript );
			break;
		case LANG_HTML:
			pLexer = new QextScintillaLexerHTML( pScript );
			break;
		case LANG_SQL:
			pLexer = new QextScintillaLexerSQL( pScript );
			break;
		case LANG_LUA:
			pLexer = new QextScintillaLexerLua( pScript );
			break;
		//case LANG_SCHEME:
			//pLexer = new QextScintillaLexerLisp( pScript );
			//break;
		case LANG_LISP:
			pLexer = new QextScintillaLexerLisp( pScript );
			break;
		case LANG_TCL:
// TODO vorlaeufig...
			pLexer = new QextScintillaLexerCPP( pScript );
			break;
		case LANG_CSS:
			pLexer = new QextScintillaLexerCSS( pScript );
			break;
		case LANG_MAKEFILE:
			pLexer = new QextScintillaLexerMakefile( pScript );
			break;
		case LANG_BATCH:
			pLexer = new QextScintillaLexerBatch( pScript );
			break;
		case LANG_BASH:
			pLexer = new QextScintillaLexerBash( pScript );
			break;
		case LANG_TEX:
			pLexer = new QextScintillaLexerTeX( pScript );
			break;
		default:
			pLexer = 0;
	}

//old:	UpdateFont();

	pScript->setLexer( pLexer, iActLanguageID );

	UpdateFont( pScript, 0 );

	if( pLexer )
	{
		pLexer->refreshProperties();
	}
	// generate a repaint() so that the lexer colorizes the source code again
	if( !pScript->text().isEmpty() )
	{
		pScript->setText( pScript->text() );
		// by just repainting the ui control nothing changed at the data !
		pScript->setEdited( false );
	}

	pScript->update();
	pScript->repaint();
}

static QString GetCommentTokenForLine( int iLanguageID )
{
	QString sRet;

	switch( iLanguageID )
	{
		case LANG_NONE:
			break;
		case LANG_MINSCRIPT:
		case LANG_CPP:
		case LANG_CSHARP:
		case LANG_JAVA:
		case LANG_JAVASCRIPT:
		case LANG_PHP:
		case LANG_IDL:
			sRet = "//";
			break;
		case LANG_MAKEFILE:
		case LANG_PERL:
		case LANG_PYTHON:
		case LANG_TCL:
		case LANG_BASH:
		case LANG_RUBY:
		case LANG_LUA:		// ?
			sRet = "#";
			break;
		case LANG_BATCH:
			sRet = "REM ";
			break;
		//case LANG_SCHEME:
		case LANG_LISP:
			sRet = ";;";
			break;
		case LANG_HTML:
			sRet = "<!--";	// has to be closed with -->
			break;
		case LANG_SQL:
			sRet = "--";
			break;
		case LANG_TEX:
			sRet = "%";
			break;
		case LANG_CSS:
			sRet = "//";	// --> eigentlich /* */
			break;
		default:
			sRet = "";
	}

	return sRet;
}

void VisiScript::UpdateLanguage( bool bForAllDocuments, int iActLanguageID )
{
	if( bForAllDocuments )
	{
		for( int i=0; i<GetDocumentCount(); i++ )
		{
			MyQCodeEdit * pScript = /*GetActScript();*/ GetScriptFor( i );
			UpdateLanguage( pScript, iActLanguageID );
		}
	}
	else
	{
		MyQCodeEdit * pScript = GetActScript();
		UpdateLanguage( pScript, iActLanguageID );
	}
}

void VisiScript::UpdateLanguageMenu()
{
	// deselect all languages
	for( int i=LANG_NONE; i<LANG_MAX_ID; i++ )
	{
		m_pLanguages->setItemChecked( i, false );
	}
	// and select the new language
	m_pLanguages->setItemChecked( GetActScript()->languageID(), true );
}

void VisiScript::UpdateScriptOutputRatio()
{
	sltMaximizeOutput();
}

void VisiScript::InsertInSelection( bool bDoInsert, const QString & sToken )
{
	MyQCodeEdit * pScript = GetActScript();

	if( pScript && pScript->hasSelectedText() )
	{
		int iFromLine,iFromPos,iToLine,iToPos;

		pScript->getSelection( &iFromLine, &iFromPos, &iToLine, &iToPos );

		int iTokenLen = sToken.length();

		pScript->beginUndoAction();
		for( int i=iFromLine; i<=iToLine; i++ )
		{
			int iStartIndex = 0;
			if( i==iFromLine )
			{
				iStartIndex = iFromPos;
			}
			if( i==iToLine && iToPos==0 )
			{
				break;
			}
			if( bDoInsert )
			{
				pScript->insertAt( sToken, i, iStartIndex );
			}
			else
			{
				QString s = pScript->text( i );
				if( s.mid( iStartIndex, iTokenLen ) == sToken )
				{
					pScript->setSelection( i, iStartIndex, i, iStartIndex+iTokenLen );
					pScript->removeSelectedText();
				}
			}
		}
		pScript->endUndoAction();

		pScript->setSelection( iFromLine, iFromPos, iToLine, iToPos );
	}
}

int VisiScript::GetHeightForOutput( int iPercent )
{
	int iValue = height()*iPercent/100;
	return iValue;
}

void VisiScript::PushLastFileName( const QString & sName )
{
	PushToList( sName, m_aLastFilesList, MAX_LAST_FILES );
/*
	if( !ExistsLastFileName( sName ) )
	{
		m_aLastFilesList.append( new QString( sName ) );

		if( m_aLastFilesList.count()>MAX_LAST_FILES )
		{
			m_aLastFilesList.removeFirst();
		}
	}
*/
}

bool VisiScript::ExistsLastFileName( const QString & sName )
{
	return ExistsInList( sName, m_aLastFilesList );
/*
	for( unsigned int i=0; i<m_aLastFilesList.count(); i++ )
	{
		if( *(m_aLastFilesList.at( i )) == sName )
		{
			return true;
		}
	}

	return false;
*/
}

bool VisiScript::DoImportFile( const QString & sFileName, int x, int y )
{
	if( !sFileName.isNull() )
	{
		QString sText;
		bool bOk = ReadTextFile( sFileName/*.latin1()*/, sText, m_bSetUtf8 );

		if( bOk )
		{
			AddDocument( -1, sFileName );

	    	GetActScript()->setText( sText );
    		GetActScript()->setEdited( FALSE );
			GetActScript()->setCursorPosition(y,x);
			//updateCaption( sFileName+" #" );

			PushLastFileName( sFileName/*.latin1()*/ );

			return true;
		}
	}
	return false;
}

bool VisiScript::DoExportFile( const QString & sFileName, bool bSaveOutput )
{
	QString sText;
	if( bSaveOutput )
	{
		sText = GetActOutput()->text();
	}
	else
	{
		sText = GetActScript()->text();
	}

	bool bOk = WriteTextFile( sFileName/*.latin1()*/, /*m_bSetUtf8 ? sText.utf8() : sText.latin1()*/sText, /*append=*/false, m_bSetUtf8 );

	if( bOk && !bSaveOutput )
	{
    	GetActScript()->setEdited(FALSE);
		//updateCaption( sFileName+" #" );
	}

	return bOk;
}

bool VisiScript::DoExportFile( MyQCodeEdit * pScript )
{
	QString sText = pScript->text();

	bool bOk = WriteTextFile( pScript->fileName()/*.latin1()*/, /*m_bSetUtf8 ? sText.utf8() : sText.latin1()*/sText, /*append=*/false, m_bSetUtf8 );

	if( bOk )
	{
    	pScript->setEdited(FALSE);
	}

	return bOk;
}

void VisiScript::SetOutputSize( int iPercentSize )
{
#ifdef ZAURUS
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		GetOutputFor( i )->setMinimumSize( GetOutputFor( i )->minimumSize().width(), GetHeightForOutput( iPercentSize ) );
		GetOutputFor( i )->setMaximumSize( GetOutputFor( i )->maximumSize().width(), GetHeightForOutput( iPercentSize ) );
	}
#endif
}

void VisiScript::sltEditUndo()
{
	GetActScript()->undo();

	GetActScript()->setEdited( GetActScript()->isModified() );
}

void VisiScript::sltEditRedo()
{
	GetActScript()->redo();

	GetActScript()->setEdited( GetActScript()->isModified() );
}

void VisiScript::sltEditCut()
{
	if( GetActScript()->hasMarkedTxt() )
	{
		GetActScript()->cut();
	}
}

void VisiScript::sltEditCopy()
{
	if( GetActScript()->hasMarkedTxt() )
	{
		GetActScript()->copy();
	}
	else if( GetActOutput()->hasMarkedTxt() )
	{
		GetActOutput()->copy();
	}
}

void VisiScript::sltEditPaste()
{
	GetActScript()->paste();
}

void VisiScript::sltClearOutput()
{
	GetActOutput()->setText( "" );
}

void VisiScript::sltCommentSelection()
{
	QString sCommentToken = GetCommentTokenForLine( GetActScript()->languageID() );
	InsertInSelection( true, sCommentToken );
}

void VisiScript::sltUnCommentSelection()
{
	QString sCommentToken = GetCommentTokenForLine( GetActScript()->languageID() );
	InsertInSelection( false, sCommentToken );
}

QString VisiScript::GetStringForTab() const
{
	if( m_iSpacesForTabs )
	{
		QString s;
		/*s =*/ s.fill( ' ', m_iTabSpace );
		return s;
	}
	else
	{
		return "\t";
	}
}

void VisiScript::sltTabWithSelectedTextPressed()
{
	MyQCodeEdit * pScript = GetActScript();

	if( pScript && pScript->hasSelectedText() )
	{
		InsertInSelection( true, GetStringForTab() );
	}
}

void VisiScript::sltShiftTabWithSelectedTextPressed()
{
	MyQCodeEdit * pScript = GetActScript();

	if( pScript && pScript->hasSelectedText() )
	{
		InsertInSelection( false, GetStringForTab() );
	}
}

void VisiScript::sltAddLineToOutput( const QString & sLine )
{
	//GetActOutput()->setText( GetActOutput()->text() + sLine + "\n" );
	// should fix the scroll problem (it does not work for the Zaurus...)
	GetActOutput()->insertLine( sLine );

	// hmm, das loest wohl das output scrolling problem ?
	GetActOutput()->setCursorPosition( GetActOutput()->numLines(), 1 );
}

bool VisiScript::doFindFirst( int iLine, int iIndex )
{
	bool bFound = GetActScript()->findFirst( m_sFindText,
		 					   m_bRegularExpr,
							   m_bCaseSensitive,
							   m_bOnlyWholeWords,
							   /*bWrap*/false,
							   !m_bSearchBackward, 
							   iLine, iIndex );

	if( !bFound )
	{
		/*int iRet =*/ QMessageBox::information( this, tr( "Find Text" ), tr( "Text was not found!" ), QMessageBox::Ok );
	}

	return bFound;
}

void VisiScript::sltEditFindText()
{
	if( m_pFindDlg==0 )
	{
		m_pFindDlg =
#ifdef ZAURUS
	/*FindDlg * pDlg =*/ new FindDlg( this, "findtextdlg", /*bModal*/TRUE );
#else
	/*FindDlgImpl * pDlg =*/ new FindDlgImpl( this, "findtextdlg", /*bModal*/TRUE );
#endif
		m_pFindDlg->adjustSize();
	}

	FindDlg * pDlg = m_pFindDlg;

	// update the combo box for the old search strings
	pDlg->m_pFindText->clear();
	for( unsigned int i=0; i<m_aLastSearchStringList.count(); i++ )
	{
		pDlg->m_pFindText->insertItem( *(m_aLastSearchStringList.at( i )), 0 );
	}

	// if any text is marked, use this text for searching
	if( GetActScript()->hasMarkedTxt() )
	{
		pDlg->m_pFindText->setEditText( GetActScript()->markedTxt() );
	}
	else
	{
		pDlg->m_pFindText->setEditText( m_sFindText );
	}
	pDlg->m_pRegularExpr->setChecked( m_bRegularExpr );
	pDlg->m_pCaseSensitive->setChecked( m_bCaseSensitive );
	pDlg->m_pOnlyWholeWord->setChecked( m_bOnlyWholeWords );
	pDlg->m_pSearchBackward->setChecked( m_bSearchBackward );

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_bIsReplace = false;

		m_sFindText = pDlg->m_pFindText->currentText();
		m_bRegularExpr = pDlg->m_pRegularExpr->isChecked();
		m_bCaseSensitive = pDlg->m_pCaseSensitive->isChecked();
		m_bOnlyWholeWords = pDlg->m_pOnlyWholeWord->isChecked();
		m_bSearchBackward = pDlg->m_pSearchBackward->isChecked();

		// update the list of the last searched items
		PushToList( m_sFindText, m_aLastSearchStringList, MAX_LAST_SEARCH_STRINGS );

		doFindFirst();
	}

	//delete pDlg;
}

void VisiScript::sltEditFindNext()
{
	bool bFound = GetActScript()->findNext();

	if( m_bIsReplace && bFound )
	{
		GetActScript()->replace( m_sReplaceText );
	}

	if( !bFound )
	{
		int iRet = QMessageBox::information( this, tr( "Find Text" ), tr( "Searched to the end of the text.\nStart search again?" ), QMessageBox::Yes, QMessageBox::No );

		if( iRet == QMessageBox::Yes )
		{
			bFound = doFindFirst( 0, 0 );

			if( m_bIsReplace && bFound )
			{
				GetActScript()->replace( m_sReplaceText );
			}
		}
	}
}

void VisiScript::sltEditReplaceText()
{
	if( m_pReplaceDlg==0 )
	{
		m_pReplaceDlg =
#ifdef ZAURUS
	/*ReplaceDlg * pDlg =*/ new ReplaceDlg( this, "replacetextdlg", /*bModal*/TRUE );
#else
	/*ReplaceDlgImpl * pDlg =*/ new ReplaceDlgImpl( this, "replacetextdlg", /*bModal*/TRUE );
#endif
		m_pReplaceDlg->adjustSize();
	}

	ReplaceDlg * pDlg = m_pReplaceDlg;

	// update the combo box for the old search strings
	pDlg->m_pFindText->clear();
	pDlg->m_pReplaceText->clear();
	unsigned int i = 0;
	for( i=0; i<m_aLastSearchStringList.count(); i++ )
	{
		pDlg->m_pFindText->insertItem( *(m_aLastSearchStringList.at( i )), 0 );
	}
	for( i=0; i<m_aLastReplaceStringList.count(); i++ )
	{
		pDlg->m_pReplaceText->insertItem( *(m_aLastReplaceStringList.at( i )), 0 );
	}

	// if any text is marked, use this text for searching
	if( GetActScript()->hasMarkedTxt() )
	{
		pDlg->m_pFindText->setEditText( GetActScript()->markedTxt() );
	}
	else
	{
		pDlg->m_pFindText->setEditText( m_sFindText );
	}
	pDlg->m_pReplaceText->setEditText( m_sReplaceText );
	pDlg->m_pRegularExpr->setChecked( m_bRegularExpr );
	pDlg->m_pCaseSensitive->setChecked( m_bCaseSensitive );
	pDlg->m_pOnlyWholeWord->setChecked( m_bOnlyWholeWords );
	pDlg->m_pSearchBackward->setChecked( m_bSearchBackward );
	pDlg->m_pReplaceAll->setChecked( m_bReplaceAll );

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_bIsReplace = true;

		m_sFindText = pDlg->m_pFindText->currentText();
		m_sReplaceText = pDlg->m_pReplaceText->currentText();
		m_bRegularExpr = pDlg->m_pRegularExpr->isChecked();
		m_bCaseSensitive = pDlg->m_pCaseSensitive->isChecked();
		m_bOnlyWholeWords = pDlg->m_pOnlyWholeWord->isChecked();
		m_bSearchBackward = pDlg->m_pSearchBackward->isChecked();
		m_bReplaceAll = pDlg->m_pReplaceAll->isChecked();

		// update the list of the last searched items
		PushToList( m_sFindText, m_aLastSearchStringList, MAX_LAST_SEARCH_STRINGS );
		PushToList( m_sReplaceText, m_aLastReplaceStringList, MAX_LAST_SEARCH_STRINGS );

		bool bFound = doFindFirst();

		if( bFound )
		{
			GetActScript()->beginUndoAction();

			GetActScript()->replace( m_sReplaceText );

			while( m_bReplaceAll && bFound )
			{
				bFound = GetActScript()->findNext();

				if( m_bIsReplace && bFound )
				{
					GetActScript()->replace( m_sReplaceText );
				}
			}

			GetActScript()->endUndoAction();
		}
	}

	//delete pDlg;
}

static void inc_if( int * pValue )
{
	if( pValue )
	{
		*pValue += 1;
	}
}

static QString singleFileSearch( const QString & strFileName, 
								 const QString & strSearch, 
								 bool bCaseSensitive=true, 
								 bool bRegExpr=true, 
								 bool bWildcard=false, 
								 int * pFoundCount=0, 
								 const QString & strFileTag = "", 
								 const QString & strLineTag = "",
								 QWidget * pObserver = 0 )
{
	QString strRet;

	QFile aFile( strFileName );
	if( !aFile.open(IO_ReadOnly | IO_Translate) )
	{
		strRet = "Error: can not open file " + strFileName;
	}
	else
	{
		QTextStream aInStream( &aFile );
		//aInStream.setEncoding(QTextStream::UnicodeUTF8);

		//QString strWholeFile = aInStream.read();

		QRegExp * pRegExpr = 0;
		if( bRegExpr )
		{
			pRegExpr = new QRegExp( strSearch, bCaseSensitive, bWildcard );
		}

		int iCount = 0;
		while( !aInStream.atEnd() )
		{
			QString strLine = aInStream.readLine();
			iCount++;

			int iFoundPos = -1;
			if( bRegExpr && pRegExpr )
			{
#ifndef ZAURUS
				iFoundPos = pRegExpr->search( strLine );
#else
				iFoundPos = pRegExpr->find( strLine, 0 );
#endif
			}
			else
			{
				iFoundPos = strLine.find( strSearch, 0, bCaseSensitive );
			}
			if( iFoundPos >= 0 )
			{
				// found !
				QString strTemp = strFileTag + strFileName + "; " + strLineTag + QString::number( iCount ) + ": " + strLine; // + "\n";

				if( pObserver )
				{
					QString * pString = new QString( strTemp );
					QCustomEvent * pEvent = new QCustomEvent((QEvent::Type)_USER_EVENT_MSG,pString);
					QApplication::postEvent(pObserver,pEvent);
				}

				strRet += strTemp;

				// count all occurences in this line !
				while( iFoundPos >= 0 )
				{
					inc_if( pFoundCount );

					if( bRegExpr && pRegExpr )
					{
#ifndef ZAURUS
						iFoundPos = pRegExpr->search( strLine, iFoundPos+1/*pRegExpr->matchedLength()*/ );
#else
						iFoundPos = pRegExpr->find( strLine, iFoundPos+1/*pRegExpr->matchedLength()*/ );
#endif
					}
					else
					{
						iFoundPos = strLine.find( strSearch, iFoundPos+1, bCaseSensitive );
					}
				}
			}
		}
		//strRet += strFileName + " count=" + QString::number(iCount) + "\n";

		if( pRegExpr )
		{
			delete pRegExpr;
		}
	}

	return strRet;
}

static pair<bool,QString> recursiveFileSearch( const QString & strPath, 
								    const QString & strFiles,
								    const QString & strSearch, 
								    bool bCaseSensitive=true, 
									bool bSerachInSubDirs=true,
									bool bRegExpr=true,
									bool bWildcard=false,
									int * pFoundCount=0, 
									int * pFindFileCount=0, 
									int * pTotalCount=0,
									const QString & strFileTag = "", 
								    const QString & strLineTag = "",
									QWidget * pObserver = 0,
									bool * pStopFlag = 0 )
{
	QString strRet;
	QDir aDir(strPath);
	QStringList::Iterator it;

	QStringList aFiles = aDir.entryList(strFiles,QDir::Files);

	it = aFiles.begin();
	while( it != aFiles.end() ) 
	{
		QString strName = QFileInfo(strPath, *it).absFilePath();
		QString strFound = singleFileSearch( strName, strSearch, bCaseSensitive, bRegExpr, bWildcard, pFoundCount, strFileTag, strLineTag, pObserver );

		// handle the stopping flag...
		if( pStopFlag && *pStopFlag )
		{
			return pair<bool,QString>(false,strRet);
		}

		inc_if( pTotalCount );

		if( !strFound.isNull() && !strFound.isEmpty() )
		{
			strRet += strFound;			
			//strRet += "\n";
			inc_if( pFindFileCount );
		}

		++it;
	}

	if( bSerachInSubDirs )
	{
		QStringList dirs = aDir.entryList(QDir::Dirs);
		it = dirs.begin();
		while( it != dirs.end() )
		{
			if( *it != "." && *it != ".." )
			{
				pair<bool,QString> aRet = recursiveFileSearch( strPath + "/" + *it, strFiles, strSearch, bCaseSensitive, bSerachInSubDirs, bRegExpr, bWildcard, pFoundCount, pFindFileCount, pTotalCount, strFileTag, strLineTag, pObserver, pStopFlag );
				strRet += aRet.second;
				if( !aRet.first )
				{
					return pair<bool,QString>(aRet.first,strRet);
				}
			}
			++it;
		}
	}

	return pair<bool,QString>(true,strRet);
}

#ifdef _with_thread_search

#ifdef ZAURUS
void * searchThreadStart( void * pData );
#endif

class MyFindInFiles 
#ifndef ZAURUS
	: public QThread
#endif
{
public:
	MyFindInFiles(  const QString & strPath, 
					const QString & strFiles,
					const QString & strSearch, 
					bool bCaseSensitive=true, 
					bool bSerachInSubDirs=true,
					bool bRegExpr=true,
					bool bWildcard=false,
					int * pFoundCount=0, 
					int * pFindFileCount=0, 
					int * pTotalCount=0,
					const QString & strFileTag = "", 
					const QString & strLineTag = "",
					QWidget * pObserver = 0,
					bool * pStopFlag = 0 )
		: m_strPath( strPath )
		, m_strFiles( strFiles ) 
		, m_strSearch( strSearch )
		, m_bCaseSensitive( bCaseSensitive )
		, m_bSerachInSubDirs( bSerachInSubDirs )
		, m_bRegExpr( bRegExpr )
		, m_bWildcard( bWildcard )
		, m_pFoundCount( pFoundCount )
		, m_pFindFileCount( pFindFileCount )
		, m_pTotalCount( pTotalCount )
		, m_strFileTag( strFileTag )
		, m_strLineTag( strLineTag )
		, m_pObserver( pObserver )
		, m_pStopFlag( pStopFlag )
	{
	}

    virtual void run()
	{
		pair<bool,QString> aRet = recursiveFileSearch( m_strPath, 
								    m_strFiles,
								    m_strSearch, 
								    m_bCaseSensitive, 
									m_bSerachInSubDirs,
									m_bRegExpr,
									m_bWildcard,
									m_pFoundCount, 
									m_pFindFileCount, 
									m_pTotalCount,
									m_strFileTag, 
								    m_strLineTag,
									m_pObserver,
									m_pStopFlag );
		QString strLastMsg = ""+QObject::tr("found: ")+QString::number(*m_pFoundCount)+QObject::tr(" in files: ")+QString::number(*m_pFindFileCount)+QObject::tr(", total files: ")+QString::number(*m_pTotalCount); 
		if( m_pObserver )
		{
			QString strResult;
			if( !aRet.first )
			{
				strResult = QObject::tr("search stoped !\n");
			}
			QString * pString = new QString( strResult+strLastMsg );
			QCustomEvent * pEvent = new QCustomEvent((QEvent::Type)_USER_EVENT_MSG,pString);
			QApplication::postEvent(m_pObserver,pEvent);

			QCustomEvent * pEvent2 = new QCustomEvent((QEvent::Type)_USER_EVENT_DONE,0);
			QApplication::postEvent(m_pObserver,pEvent2);
		}
	}

    void stop()
	{
	}

#ifdef ZAURUS
	void start()
	{
		pthread_t aThread;
		/*int nRet =*/ pthread_create( &aThread, NULL, &searchThreadStart, (void *)this );
	}
#endif

private:
	QString		m_strPath;
	QString		m_strFiles;
	QString		m_strSearch;
	bool		m_bCaseSensitive; 
	bool		m_bSerachInSubDirs;
	bool		m_bRegExpr;
	bool		m_bWildcard;
	int *		m_pFoundCount;
	int *		m_pFindFileCount;
	int *		m_pTotalCount;
	QString		m_strFileTag;
	QString		m_strLineTag;
	QWidget *	m_pObserver;
	bool *		m_pStopFlag;
};

#ifdef ZAURUS
void * searchThreadStart( void * pData )
{
	MyFindInFiles * pThread = (MyFindInFiles *)pData;

	if( pThread )
	{
		pThread->run();
	}

	return 0;
}
#endif
#endif

void VisiScript::sltEditFindInFiles()
{
#ifdef _with_thread_search
	if( m_pFindThread!=0 )
	{
		// falls eine Suche laueft, diese beenden !
		m_bStopFlag = true;
	}
	else
	{
#endif
	const QString g_strWhitespaces = "[ \t;:.-+*/=<>?!(){}\\[]";		// whitespaces and separators \[\]\{\}

	if( m_pFindInFilesDlg==0 )
	{
		m_pFindInFilesDlg = new FindInFilesDlgImpl( this, "findinfilestextdlg", /*bModal*/TRUE );

		m_pFindInFilesDlg->adjustSize();
	}

	FindInFilesDlgImpl * pDlg = m_pFindInFilesDlg;

	// update the combo box for the old search strings
	pDlg->m_pFindText->clear();
	pDlg->m_pExtensionText->clear();
	pDlg->m_pDirectoryText->clear();
	unsigned int i = 0;
	for( i=0; i<m_aLastSearchStringList.count(); i++ )
	{
		pDlg->m_pFindText->insertItem( *(m_aLastSearchStringList.at( i )), 0 );
	}
	// update the combo box for the old files strings
	for( i=0; i<m_aLastSearchFilesList.count(); i++ )
	{
		pDlg->m_pExtensionText->insertItem( *(m_aLastSearchFilesList.at( i )), 0 );
	}
	// update the combo box for the old directory strings
	for( i=0; i<m_aLastSearchDirList.count(); i++ )
	{
		pDlg->m_pDirectoryText->insertItem( *(m_aLastSearchDirList.at( i )), 0 );
	}

	// if any text is marked, use this text for searching
	if( GetActScript()->hasMarkedTxt() )
	{
		pDlg->m_pFindText->setEditText( GetActScript()->markedTxt() );
	}
	else
	{
		pDlg->m_pFindText->setEditText( m_sFindText );	
	}

	pDlg->m_pDirectoryText->setEditText(m_sSearchDir);
	pDlg->m_pExtensionText->setEditText(m_sSearchFiles);
	pDlg->m_pRegularExpr->setChecked( m_bRegularExpr );
	pDlg->m_pWildcard->setChecked( m_bWildcard );
	pDlg->m_pCaseSensitive->setChecked( m_bCaseSensitive );
	pDlg->m_pOnlyWholeWord->setChecked( m_bOnlyWholeWords );
	pDlg->m_pSerachInSubDirs->setChecked( m_bSearchSubDirectories );

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		m_sFindText = pDlg->m_pFindText->currentText();
		m_sSearchFiles = pDlg->m_pExtensionText->currentText();
		m_sSearchDir = pDlg->m_pDirectoryText->currentText();
		m_bRegularExpr = pDlg->m_pRegularExpr->isChecked();
		m_bWildcard = pDlg->m_pWildcard->isChecked();
		m_bCaseSensitive = pDlg->m_pCaseSensitive->isChecked();
		m_bOnlyWholeWords = pDlg->m_pOnlyWholeWord->isChecked();
		m_bSearchSubDirectories = pDlg->m_pSerachInSubDirs->isChecked();

		// update the list of the last searched items
		PushToList( m_sFindText, m_aLastSearchStringList, MAX_LAST_SEARCH_STRINGS );
		PushToList( m_sSearchFiles, m_aLastSearchFilesList, MAX_LAST_SEARCH_STRINGS );
		PushToList( m_sSearchDir, m_aLastSearchDirList, MAX_LAST_SEARCH_STRINGS );

		if( !m_sFindText.isNull() && !m_sFindText.isEmpty() )
		{
#ifdef _with_thread_search
			m_iCount = 0;
			m_iFoundFileCount = 0;
			m_iTotalFileCount = 0;
#else
			int iCount = 0;
			int iFoundFileCount = 0;
			int iTotalFileCount = 0;
#endif
			GetActOutput()->append( tr("searching for: ")+"\""+m_sFindText+"\"" ); 

			bool bRegularExpr = m_bRegularExpr;
			QString sFindText = m_sFindText;
			// handle bOnlyWholeWords as regular expression !
			if( m_bOnlyWholeWords )
			{
				bRegularExpr = true;
				sFindText = g_strWhitespaces+m_sFindText+g_strWhitespaces;
			}

#ifdef _with_thread_search
			if( m_pFindThread==0 )
			{
				m_bStopFlag = false;
				m_pFindThread = new MyFindInFiles( m_sSearchDir, m_sSearchFiles, sFindText, m_bCaseSensitive, m_bSearchSubDirectories, bRegularExpr, m_bWildcard, &m_iCount, &m_iFoundFileCount, &m_iTotalFileCount, c_strFileTag, c_strLineTag, this, &m_bStopFlag );
				m_pFindThread->start();
				// WARNING: wenn das Ausgabe-Fenster geschlossen wir, dann sturzt das Programm ab !!!
				// TODO min --> robustere Behandlung der Ausgabe durchfuehren !!!
				m_pFindOutputWindow = GetActOutput();
			}
#else
			QString strFound = recursiveFileSearch(m_sSearchDir, m_sSearchFiles, sFindText, m_bCaseSensitive, m_bSearchSubDirectories, bRegularExpr, m_bWildcard, &iCount, &iFoundFileCount, &iTotalFileCount, c_strFileTag, c_strLineTag, this );
			GetActOutput()->append( strFound ); 
			GetActOutput()->append( ""+tr("found: ")+QString::number(iCount)+tr(" in files: ")+QString::number(iFoundFileCount)+tr(", total files: ")+QString::number(iTotalFileCount) ); 
			// set cursor at the end of the output...
			GetActOutput()->setCursorPosition( GetActOutput()->numLines(), 1 );
#endif
		}
		else
		{
			GetActOutput()->append( tr("Error: can not search for empty string !") ); 
		}
	}

	//delete pDlg;

#ifdef _with_thread_search
	}	// else
#endif
}

void VisiScript::customEvent(QCustomEvent * pEvent)
{
#ifdef _with_thread_search
	if( pEvent && pEvent->type()==_USER_EVENT_MSG )
	{
		QString * pStrFound = (QString *)pEvent->data();

		if( m_pFindOutputWindow )
		{
			m_pFindOutputWindow->append( *pStrFound ); 
			// set cursor at the end of the output...
			m_pFindOutputWindow->setCursorPosition( m_pFindOutputWindow->numLines(), 0 );
#ifndef ZAURUS
			//m_pFindOutputWindow->ensureCursorVisible();	// funktioniert wohl nicht !
			m_pFindOutputWindow->scrollToBottom();			
#endif
		}
		delete pStrFound;
	}
	else if( pEvent && pEvent->type()==_USER_EVENT_DONE )
	{
		if( m_pFindThread )
		{
            if( m_pFindOutputWindow )
            {   
                m_pFindOutputWindow->setCursorPosition( m_pFindOutputWindow->numLines(), 0 );       
#ifndef ZAURUS
				//m_pFindOutputWindow->ensureCursorVisible();
				m_pFindOutputWindow->scrollToBottom();		// Bugfix 10.1.2006
#endif
            }
			delete m_pFindThread;
			m_pFindThread = 0;
			m_pFindOutputWindow = 0;
		}
	}
#endif
}

void VisiScript::sltEditGotoLine()
{
	bool bOk = false;
    int iActLine = 0;
	int iIndex = 0;
	GetActScript()->getCursorPosition(&iActLine,&iIndex);
	QString strLabel;
	strLabel = strLabel.sprintf(tr("Goto line (max. %d):"),GetActScript()->lines());
	QString strTitle = tr("Goto line");
#ifndef ZAURUS
	int iLine = QInputDialog::getInteger(strTitle,strLabel,iActLine+1,0,GetActScript()->lines(),1,&bOk,this);
	if( bOk )
	{
		GetActScript()->setFocus();
		GetActScript()->setCursorPosition(iLine-1,0);
	}
#else
	InputDlg * pDlg = new InputDlg( this, "inputdlg", /*bModal*/TRUE );

	pDlg->setCaption(strTitle);
	pDlg->m_pLabel->setText(strLabel);

	pDlg->m_pText->setValidator(new QIntValidator(0,GetActScript()->lines(),pDlg->m_pText));

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		QString strValue = pDlg->m_pText->text();
		int iLine = strValue.toInt(&bOk);
		if( bOk )
		{
			GetActScript()->setFocus();
			GetActScript()->setCursorPosition(iLine-1,0);
		}
	}

	delete pDlg;
#endif
}

void VisiScript::sltClearScript()
{
	if( CheckActData() )
	{
		GetActScript()->setText( "" );
		GetActScript()->setEdited(FALSE);
	}
}

static QString addKommasIfNeeded( const QString & strPath )
{
	if( strPath.find(' ')>=0 )
	{
		return "\"" + strPath + "\"";
	}
	return strPath;
}

#define MAX_BUFFER_LEN 512

void VisiScript::sltRun()
{
	DoRun( /*bUseMainScript*/true );
}

void VisiScript::sltActRun()
{
	DoRun( /*bUseMainScript*/false );
}

void VisiScript::DoRun(bool bUseMainScript)
{
	MyQCodeEdit * pRunScript = 0;
	if( bUseMainScript )
	{
		pRunScript = GetActMainScript();
	}
	else
	{
		pRunScript = GetActScript();
	}
	QString sScript		= pRunScript->text();
	bool    bModified	= pRunScript->isModified();
	QString strActDir;
	QString strScriptFileName = _TEMP_SCRIPT_FILE;
	bool	bUseTempFile = true;

	// handle auto save for script, if checked
	if( m_bAutoSave && bModified )
	{
		sltFileSave();
		bModified = pRunScript->isModified();
	}

	if( m_bAutoClearOutput )
	{
		sltClearOutput();
	}

	if( !bModified )
	{
		strScriptFileName = pRunScript->fileName();
		// we are using the original script file, do not remove this file after execution !
		bUseTempFile = false;
	}

	// do we have to change the actual directory before we execute the script ?
	if( m_bChangeActDir )
	{
		// save the actual directory
		strActDir = QDir::currentDirPath();
		// get the path to directory of the actual script
		QString strScriptActDir = QFileInfo(pRunScript->fileName()).dirPath();
		QDir::setCurrent(strScriptActDir);
		//chdir((const char *)strScriptActDir);
	}

	if( m_iExecuteMode != EXECUTE_MODE_QPROCESS )
	{
		if( !sScript.isNull() && bUseTempFile )
		{
			// write the temporary file (if neccassary)
			WriteTextFile( /*(const char *)*/strScriptFileName, /*(const char *)*/sScript, /*append=*/false, m_bSetUtf8 );
		}
	}

#ifdef __linux__ // ZAURUS
	char sCommand[MAX_BUFFER_LEN];
#endif
	const char * sScriptCmd = (const char *)m_sScriptCmd;

	QString sOutput;

	if( m_iExecuteMode == EXECUTE_MODE_POPEN )
	{
#ifdef __linux__ // ZAURUS
		sprintf( sCommand, "%s %s %s 2>%s", (sScriptCmd ? sScriptCmd : ""), (const char *)addKommasIfNeeded(strScriptFileName), (const char *)m_sScriptArguments, _TEMP_ERR_FILE );

		FILE * pPythonOutput = popen( sCommand, "r" );

		char sBuffer[MAX_BUFFER_LEN];
		size_t iSize = 0;
		do {
			iSize = fread( sBuffer, 1, MAX_BUFFER_LEN-1, pPythonOutput );
			sBuffer[iSize] = 0;
			if( iSize>0 )
			{
				sOutput += sBuffer;
			}
		} while( iSize>0 );

		pclose( pPythonOutput );
#else
		/*int iRet =*/ spawnlp(_P_WAIT,"cmd","/C",sScriptCmd,(const char *)addKommasIfNeeded(strScriptFileName),(const char *)m_sScriptArguments,_TEMP_ERR_FILE_PIPE,_TEMP_OUT_FILE_PIPE,NULL);
#endif
	}
	else if( m_iExecuteMode == EXECUTE_MODE_EXECL )
	{
#ifdef __linux__  // ZAURUS
		sprintf( sCommand, "%s %s %s 2>%s 1>%s", (sScriptCmd ? sScriptCmd : ""), (const char *)addKommasIfNeeded(strScriptFileName), (const char *)m_sScriptArguments, _TEMP_ERR_FILE, _TEMP_OUT_FILE );

		int iStatus;
		pid_t pid;

		pid = fork();
		if( pid == 0 )
		{
			execl( (const char *)m_sExeclCmd, (const char *)m_sExeclCmd, "--login", "-c", sCommand, 0 );
			exit( EXIT_FAILURE );
		}
		else if( pid < 0 )
		{
			/* fork failed ! */
			iStatus = -1;
			sltAddLineToOutput( "Error forking process." );
		}
		else
		{
			if( waitpid( pid, &iStatus, 0 ) != pid )
			{
				iStatus = -1;
			}
		}
#else
		/*int iRet =*/ spawnlp(_P_WAIT,"cmd","/C",sScriptCmd,(const char *)addKommasIfNeeded(strScriptFileName),(const char *)m_sScriptArguments,_TEMP_ERR_FILE_PIPE,_TEMP_OUT_FILE_PIPE,NULL);
#endif
	}
	else
	{
		// first stop the running process and start again !
		sltStopInteractiveModus();

		sltStartInteractiveModus( true );

		m_pProcess->writeToStdin( sScript );

		m_pProcess->closeStdin();
	}


	if( m_iExecuteMode != EXECUTE_MODE_QPROCESS )
	{
		// WARNING: in QProcess mode there are NO arguments for the script possible !!!

		// show the output of the script in any case
		if( sOutput.length()>0 )
		{
			sltAddLineToOutput( sOutput );
		}

		// and also show the error-message, if available
		QString sErrText;
		bool bOk = ReadTextFile( _TEMP_ERR_FILE, sErrText, m_bSetUtf8 );
		if( bOk && !sErrText.isNull() && sErrText.length()>0 )
		{
			sErrText = "Error: " + sErrText;
			sltAddLineToOutput( sErrText );
		}
		remove( _TEMP_ERR_FILE );

		if( 
#ifndef ZAURUS
			// for windows allways redirecting the output into files !
			true ||
#endif
			m_iExecuteMode == EXECUTE_MODE_EXECL )
		{
			// and also show the error-message, if available
			QString sOutText;
			bool bOk2 = ReadTextFile( _TEMP_OUT_FILE, sOutText, m_bSetUtf8 );
			if( bOk2 && !sOutText.isNull() && sOutText.length()>0 )
			{
				sltAddLineToOutput( sOutText );
			}
			remove( _TEMP_OUT_FILE );
		}

		GetActOutput()->setCursorPosition( GetActOutput()->numLines(), 1 );

		if( bUseTempFile )
		{
			remove( (const char *)addKommasIfNeeded(strScriptFileName) );
		}

		if( m_bChangeActDir )
		{
			QDir::setCurrent(strActDir);
			//chdir((const char *)strActDir);
		}
	}
}

bool VisiScript::CheckActData()
{
	if( GetActScript()->edited() )
	{
		int iRet = QMessageBox::information( this, tr( "Save file" ), tr( "File was modified, save now ?" ), QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel );

		if( iRet == QMessageBox::Yes )
		{
			sltFileSave();
		}
		if( iRet == QMessageBox::Cancel )
		{
			return false;
		}
	}
	return true;
}

bool VisiScript::CheckAllData()
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		MyQCodeEdit * pScript = GetScriptFor( i );

		if( pScript->edited() )
		{
			int iRet = QMessageBox::information( this, tr( "Save file" ), tr( "File %1 was modified, save now ?" ).arg(pScript->fileName()), QMessageBox::Yes, QMessageBox::No );

			if( iRet == QMessageBox::Yes )
			{
				DoExportFile( pScript );
			}
		}
	}
	return true;
}

void VisiScript::sltNewFile()
{
    sltClearScript();

    sltShowScript();
}

void VisiScript::sltShowScript()
{
	if( GetActScript() )
	{
	    GetActScript()->setFocus();
	}
}

void VisiScript::show()
{
	QMainWindow::show();

	SetOutputSize( m_iOutputRatio );
}

void VisiScript::sltFileAboutToShow()
{
}

void VisiScript::sltEditAboutToShow()
{
	bool bEnableCopy = GetActScript()->hasMarkedTxt() || GetActOutput()->hasMarkedTxt();
	m_pCopyAction->setEnabled( bEnableCopy );
	m_pCutAction->setEnabled( bEnableCopy );

	QClipboard * pClipboard = QApplication::clipboard();
	QString sTemp = pClipboard->text();
	bool bEnablePaste = !sTemp.isNull();
	m_pPasteAction->setEnabled( bEnablePaste );

#ifdef _with_thread_search
	if( m_pFindThread==0 )
	{
		m_pFindInFilesAction->setMenuText( tr( "Find in files..." ) );
	}
	else
	{
		m_pFindInFilesAction->setMenuText( tr( "Stop find in files" ) );
	}
#endif
}

void VisiScript::sltViewAboutToShow()
{
	m_pToggleOverwrite->setOn( GetActScript()->overwriteMode() );
	bool bEnable = GetActScript()->hasMarkedTxt();
	m_pCommentAction->setEnabled( bEnable );
	m_pUnCommentAction->setEnabled( bEnable );
}

void VisiScript::sltLanguagesAboutToShow()
{
	UpdateLanguageMenu();
}

void VisiScript::sltLanguageActivated( int iId )
{
	// WARNING: the sequence of the menu has to fit with the definitions of the language ids !!!
	UpdateLanguage( false, iId );
}

void VisiScript::updateCaption( const QString & sName )
{
	if( sName.isNull() )
	{
		setCaption( tr( "VisiScript" ) );
	}
	else
	{
		setCaption( sName + " - " + tr( "VisiScript" ) );
	}
}

//const char * c_sSection 				= "Script";
const char * c_sScriptCmdKey 			= "ScriptCmd";
const char * c_sScriptArgumentsKey		= "ScriptArguments";
const char * c_sFontNameKey 			= "FontName";
const char * c_sFontStyleKey 			= "FontStyle";
const char * c_sFontSizeKey 			= "FontSize";
const char * c_sWordWrapKey				= "WordWrap";
const char * c_sTabSpaceKey				= "TabSpace";
const char * c_sOverwriteKey			= "Overwrite";
const char * c_sOutputRatio				= "OutputRatio";
const char * c_sLastDirectoryKey		= "LastDirectory";
const char * c_sExeclCmdKey				= "ExeclCmd";
const char * c_sExecuteModeKey			= "ExecuteMode";
const char * c_sInteractiveModusKey		= "InteractiveModus";
const char * c_sChangeActDirKey			= "ChangeActDir";
const char * c_sAutoSaveKey				= "AutoSave";
const char * c_sShowStaruptFileDialog	= "ShowFileDialog";
const char * c_sBraceMatchingKey		= "BraceMatching";
const char * c_sAutoClearOutputKey		= "AutoClearOutput";
const char * c_sAutoCompletionFileKey	= "AutoCompletionFile";
const char * c_sAutoCompletionThresh	= "AutoCompletionThresh";
const char * c_sSpacesForTabs			= "SpacesForTabs";
const char * c_sShowWhitespaces			= "ShowWhitespaces";
const char * c_sShowLineNumbers			= "ShowLineNumbers";
const char * c_sAutoIndent				= "AutoIndent";
const char * c_sCodeFolding				= "CodeFolding";
const char * c_sEolMode					= "EolMode";
const char * c_sLastFileName			= "LastFileName";
const char * c_sLastOpendFileName		= "LastOpendFileName";
const char * c_sLastCursorPosX			= "LastCursorPosX";
const char * c_sLastActiveFileIndex		= "LastActFileIdx";
const char * c_sLastCursorPosY			= "LastCursorPosY";
const char * c_sLastMainScriptIndex		= "LastMainScriptIdx";
const char * c_sExternalProg			= "ExternalProg";
const char * c_sLastSearchString		= "LastSearchString";
const char * c_sLastReplaceString		= "LastReplaceString";
const char * c_sLastSearchFiles			= "LastSearchFiles";
const char * c_sLastSearchDirs			= "LastSearchDirs";
const char * c_sLastScriptArguments		= "LastScriptArguments";
const char * c_sSetUtf8					= "Utf8";
const char * c_sRegExpr					= "RegExpr";
const char * c_sUseWildcard				= "UseWildcard";
const char * c_sCaseSensitive			= "CaseSensitive";
const char * c_sWholeWord				= "WholeWord";
const char * c_sSearchBackward			= "SearchBackward";
const char * c_sSearchSubDirs			= "SearchSubDirs";
const char * c_sReplaceAll				= "ReplaceAll";
const char * c_sAutoFileLoad			= "AutoFileLoad";
#ifndef ZAURUS
const char * c_sLastPosX				= "LastPosX";
const char * c_sLastPosY				= "LastPosY";
const char * c_sLastPosDX				= "LastPosDX";
const char * c_sLastPosDY				= "LastPosDY";
#endif

void VisiScript::loadInit()
{
#ifdef ZAURUS
	Config aCfg( _VISISCRIPT_NAME );

	aCfg.setGroup( _VISISCRIPT_NAME );
#else
	QSettings aCfg;

	aCfg.setPath("mneuroth.de",_VISISCRIPT_NAME,QSettings::User);
	aCfg.beginGroup(_VISISCRIPT_NAME);
#endif

	//m_sScriptArguments = GetExtentionFromScriptCmd( m_sScriptCmd );

	m_sScriptCmd = aCfg.readEntry( c_sScriptCmdKey, m_sScriptCmd );
	m_sScriptArguments = aCfg.readEntry( c_sScriptArgumentsKey, m_sScriptArguments );
	m_sFontName = aCfg.readEntry( c_sFontNameKey, m_sFontName );
	m_sFontStyle = aCfg.readEntry( c_sFontStyleKey, m_sFontStyle );
	m_iFontSize = aCfg.readNumEntry( c_sFontSizeKey, m_iFontSize );
	m_iWordWrap = aCfg.readNumEntry( c_sWordWrapKey, m_iWordWrap );
	m_iOverwrite = aCfg.readNumEntry( c_sOverwriteKey, m_iOverwrite );
	m_iTabSpace = aCfg.readNumEntry( c_sTabSpaceKey, m_iTabSpace );
	m_iEolMode = aCfg.readNumEntry( c_sEolMode, m_iEolMode );
	m_iOutputRatio = aCfg.readNumEntry( c_sOutputRatio, m_iOutputRatio );
	m_iSpacesForTabs = aCfg.readNumEntry( c_sSpacesForTabs, m_iSpacesForTabs );
	m_iShowWhitespaces = aCfg.readNumEntry( c_sShowWhitespaces, m_iShowWhitespaces );
	m_iShowLineNumbers = aCfg.readNumEntry( c_sShowLineNumbers, m_iShowLineNumbers );
	m_iAutoIndent = aCfg.readNumEntry( c_sAutoIndent, m_iAutoIndent );
	m_iCodeFolding = aCfg.readNumEntry( c_sCodeFolding, m_iCodeFolding );
	m_sLastDirectory = aCfg.readEntry( c_sLastDirectoryKey, m_sLastDirectory );
	m_sExeclCmd = aCfg.readEntry( c_sExeclCmdKey, m_sExeclCmd );
	m_iExecuteMode = aCfg.readNumEntry( c_sExecuteModeKey, m_iExecuteMode );
	m_bInteractiveModus = aCfg.readNumEntry( c_sInteractiveModusKey, m_bInteractiveModus );
	m_bChangeActDir = aCfg.readNumEntry( c_sChangeActDirKey, m_bChangeActDir );
	m_bAutoSave = aCfg.readNumEntry( c_sAutoSaveKey, m_bAutoSave );
	m_bBraceMatching = aCfg.readNumEntry( c_sBraceMatchingKey, m_bBraceMatching );
	m_bAutoClearOutput = aCfg.readNumEntry( c_sAutoClearOutputKey, m_bAutoClearOutput );
	m_bSetUtf8 = aCfg.readNumEntry( c_sSetUtf8, m_bSetUtf8 );
	m_iShowStartupFileDialog = aCfg.readNumEntry( c_sShowStaruptFileDialog, m_iShowStartupFileDialog );
	m_iAutoCompletionThresh = aCfg.readNumEntry( c_sAutoCompletionThresh, m_iAutoCompletionThresh );
	m_sAutoCompletionFileName = aCfg.readEntry( c_sAutoCompletionFileKey, m_sAutoCompletionFileName );
	m_bRegularExpr = aCfg.readNumEntry( c_sRegExpr, m_bRegularExpr );
	m_bWildcard = aCfg.readNumEntry( c_sUseWildcard, m_bWildcard );
	m_bCaseSensitive = aCfg.readNumEntry( c_sCaseSensitive, m_bCaseSensitive );
	m_bOnlyWholeWords = aCfg.readNumEntry( c_sWholeWord, m_bOnlyWholeWords );
	m_bSearchBackward = aCfg.readNumEntry( c_sSearchBackward, m_bSearchBackward );
	m_bSearchSubDirectories = aCfg.readNumEntry( c_sSearchSubDirs, m_bSearchSubDirectories );
	m_bReplaceAll = aCfg.readNumEntry( c_sReplaceAll, m_bReplaceAll );
	m_bAutoFileLoad = aCfg.readNumEntry( c_sAutoFileLoad, m_bAutoFileLoad );
	m_iLastMainScriptIndex = aCfg.readNumEntry( c_sLastMainScriptIndex, -1 );
	m_iLastActFileIndex = aCfg.readNumEntry( c_sLastActiveFileIndex, 0 );

#ifndef ZAURUS
	m_iLastPosX = aCfg.readNumEntry( c_sLastPosX, m_iLastPosX );
	m_iLastPosY = aCfg.readNumEntry( c_sLastPosY, m_iLastPosY );
	m_iLastPosDX = aCfg.readNumEntry( c_sLastPosDX, m_iLastPosDX );
	m_iLastPosDY = aCfg.readNumEntry( c_sLastPosDY, m_iLastPosDY );
#endif

	unsigned int i = 0;
	unsigned int n = 0;
    bool bDone = false;
    bool bOk = true;
    while( !bDone )
    {
		char sBuffer[512];
		QString sLastOpendFileName;

		sprintf( sBuffer, "%s.%d", c_sLastOpendFileName, n );
#ifdef ZAURUS
		bOk = aCfg.hasKey( sBuffer );
		if( bOk )
		{
			sLastOpendFileName = aCfg.readEntry( sBuffer, sLastOpendFileName );
		}
#else
		sLastOpendFileName = aCfg.readEntry( sBuffer, sLastOpendFileName, &bOk );
#endif

        if( !bOk )
        {
            // after reading last opend file list, remove all entries !
            for( i=0; i<n; i++ )
            {
				sprintf( sBuffer, "%s.%d", c_sLastOpendFileName, i );
                aCfg.removeEntry( sBuffer );
				sprintf( sBuffer, "%s.%d", c_sLastCursorPosX, i );
                aCfg.removeEntry( sBuffer );
				sprintf( sBuffer, "%s.%d", c_sLastCursorPosY, i );
                aCfg.removeEntry( sBuffer );
            }
			bDone = true;
        }
        else
        {
			int x,y;
			sprintf( sBuffer, "%s.%d", c_sLastCursorPosX, n );
			x = aCfg.readNumEntry( sBuffer, 0 );
			sprintf( sBuffer, "%s.%d", c_sLastCursorPosY, n );
			y = aCfg.readNumEntry( sBuffer, 0 );
			m_aLastCursorPos.append( new QPair<int,int>(x,y) );

			PushToList( sLastOpendFileName, m_aLastOpendFilesList );
            n++;
        }
    }    
	for( i=0; i<MAX_LAST_FILES; i++ )
	{
		char sBuffer[512];
		QString sLastFileName;

		sprintf( sBuffer, "%s.%d", c_sLastFileName, i );
		sLastFileName = aCfg.readEntry( sBuffer, sLastFileName );

		if( !sLastFileName.isEmpty() )
		{
			PushLastFileName( sLastFileName );
		}
	}
	for( i=0; i<MAX_LAST_SEARCH_STRINGS; i++ )
	{
		char sBuffer[512];
		QString sText;
		QString sTextRepl;
		QString sFile;
		QString sDir;
		QString sScriptArguments;

		sprintf( sBuffer, "%s.%d", c_sLastSearchString, i );
		sText = aCfg.readEntry( sBuffer, sText );

		sprintf( sBuffer, "%s.%d", c_sLastReplaceString, i );
		sTextRepl = aCfg.readEntry( sBuffer, sTextRepl );

		sprintf( sBuffer, "%s.%d", c_sLastSearchFiles, i );
		sFile = aCfg.readEntry( sBuffer, sFile );

		sprintf( sBuffer, "%s.%d", c_sLastSearchDirs, i );
		sDir = aCfg.readEntry( sBuffer, sDir );

		sprintf( sBuffer, "%s.%d", c_sLastScriptArguments, i );
		sScriptArguments = aCfg.readEntry( sBuffer, sScriptArguments );

		if( !sText.isEmpty() )
		{
			PushToList( sText, m_aLastSearchStringList, MAX_LAST_SEARCH_STRINGS );
		}
		if( !sTextRepl.isEmpty() )
		{
			PushToList( sTextRepl, m_aLastReplaceStringList, MAX_LAST_SEARCH_STRINGS );
		}
		if( !sFile.isEmpty() )
		{
			PushToList( sFile, m_aLastSearchFilesList, MAX_LAST_SEARCH_STRINGS );
		}
		if( !sDir.isEmpty() )
		{
			PushToList( sDir, m_aLastSearchDirList, MAX_LAST_SEARCH_STRINGS );
		}
		if( !sScriptArguments.isEmpty() )
		{
			PushToList( sScriptArguments, m_aLastScriptArgumentsList, MAX_LAST_SEARCH_STRINGS );
		}
	}
	for( i=0; i<MAX_EXTERNAL_PROGRAMS; i++ )
	{
		char sBuffer[512];
		QString sExternalProg;

		sprintf( sBuffer, "%s.%d", c_sExternalProg, i );
		sExternalProg = aCfg.readEntry( sBuffer, sExternalProg );

		if( !sExternalProg.isEmpty() )
		{
			PushToList( sExternalProg, m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		}
	}
	if( m_aExternalProgsList.count()==0 )
	{
		// create default values for first start
		PushToList( "chicken", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "scheme48", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "lush", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "lua", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "tclsh", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "ruby", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "perl", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "python -i", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "python", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
		PushToList( "minscript", m_aExternalProgsList, MAX_EXTERNAL_PROGRAMS );
	}
#ifndef ZAURUS
	aCfg.endGroup();
#endif
}

void VisiScript::saveInit()
{
#ifdef ZAURUS
	Config aCfg( _VISISCRIPT_NAME );

	aCfg.setGroup( _VISISCRIPT_NAME );
#else
	QSettings aCfg;

	aCfg.setPath("mneuroth.de",_VISISCRIPT_NAME,QSettings::User);
	aCfg.beginGroup(_VISISCRIPT_NAME);
#endif

	aCfg.writeEntry( c_sScriptCmdKey, m_sScriptCmd );
	aCfg.writeEntry( c_sScriptArgumentsKey, m_sScriptArguments );
	aCfg.writeEntry( c_sFontNameKey, m_sFontName );
	aCfg.writeEntry( c_sFontStyleKey, m_sFontStyle );
	aCfg.writeEntry( c_sFontSizeKey, m_iFontSize );
	aCfg.writeEntry( c_sWordWrapKey, m_iWordWrap );
	aCfg.writeEntry( c_sOverwriteKey, (int)GetActScript()->overwriteMode()/*m_iOverwrite*/ );
	aCfg.writeEntry( c_sTabSpaceKey, m_iTabSpace );
	aCfg.writeEntry( c_sEolMode, m_iEolMode );
	aCfg.writeEntry( c_sOutputRatio, m_iOutputRatio );
	aCfg.writeEntry( c_sSpacesForTabs, m_iSpacesForTabs );
	aCfg.writeEntry( c_sShowWhitespaces, m_iShowWhitespaces );
	aCfg.writeEntry( c_sShowLineNumbers, m_iShowLineNumbers );
	aCfg.writeEntry( c_sAutoIndent, m_iAutoIndent );
	aCfg.writeEntry( c_sCodeFolding, m_iCodeFolding );
	aCfg.writeEntry( c_sLastDirectoryKey, m_sLastDirectory );
	aCfg.writeEntry( c_sExeclCmdKey, m_sExeclCmd );
	aCfg.writeEntry( c_sExecuteModeKey, m_iExecuteMode );
	aCfg.writeEntry( c_sInteractiveModusKey, (int)m_bInteractiveModus );
	aCfg.writeEntry( c_sChangeActDirKey, (int)m_bChangeActDir );
	aCfg.writeEntry( c_sAutoSaveKey, (int)m_bAutoSave );
	aCfg.writeEntry( c_sBraceMatchingKey, (int)m_bBraceMatching );
	aCfg.writeEntry( c_sAutoClearOutputKey, (int)m_bAutoClearOutput );
	aCfg.writeEntry( c_sSetUtf8, (int)m_bSetUtf8 );
	aCfg.writeEntry( c_sShowStaruptFileDialog, m_iShowStartupFileDialog );
	aCfg.writeEntry( c_sAutoCompletionThresh, m_iAutoCompletionThresh );
	aCfg.writeEntry( c_sAutoCompletionFileKey, m_sAutoCompletionFileName );
	aCfg.writeEntry( c_sRegExpr, (int)m_bRegularExpr );
	aCfg.writeEntry( c_sUseWildcard, (int)m_bWildcard );
	aCfg.writeEntry( c_sCaseSensitive, (int)m_bCaseSensitive );
	aCfg.writeEntry( c_sWholeWord, (int)m_bOnlyWholeWords );
	aCfg.writeEntry( c_sSearchBackward, (int)m_bSearchBackward );
	aCfg.writeEntry( c_sSearchSubDirs, (int)m_bSearchSubDirectories );
	aCfg.writeEntry( c_sReplaceAll, (int)m_bReplaceAll );
	aCfg.writeEntry( c_sLastActiveFileIndex, m_pTabWidget->currentPageIndex() );
	aCfg.writeEntry( c_sAutoFileLoad, (int)m_bAutoFileLoad );
	
	int iFoundIndex = -1;
	GetActMainScript( &iFoundIndex );
	aCfg.writeEntry( c_sLastMainScriptIndex, iFoundIndex );

#ifndef ZAURUS
	m_iLastPosX = x();
	m_iLastPosY = y();
	m_iLastPosDX = width();
	m_iLastPosDY = height();
	aCfg.writeEntry( c_sLastPosX, m_iLastPosX );
	aCfg.writeEntry( c_sLastPosY, m_iLastPosY );
	aCfg.writeEntry( c_sLastPosDX, m_iLastPosDX );
	aCfg.writeEntry( c_sLastPosDY, m_iLastPosDY );
#endif

	unsigned int i = 0;
	for( i=0; i<(unsigned int)GetDocumentCount(); i++ )
    {
		char sBuffer[512];
		int x,y;
		MyQCodeEdit * pScript = GetScriptFor( i );
		pScript->getCursorPosition( &y, &x );

		QString sLastOpendFileName = pScript->fileName();
		sprintf( sBuffer, "%s.%d", c_sLastOpendFileName, i );
		aCfg.writeEntry( sBuffer, sLastOpendFileName );

		sprintf( sBuffer, "%s.%d", c_sLastCursorPosX, i );
		aCfg.writeEntry( sBuffer, x );
		sprintf( sBuffer, "%s.%d", c_sLastCursorPosY, i );
		aCfg.writeEntry( sBuffer, y );		
    }
	for( i=0; i<m_aLastFilesList.count(); i++ )
	{
		char sBuffer[512];
		QString sLastFileName = *(m_aLastFilesList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sLastFileName, i );
		aCfg.writeEntry( sBuffer, sLastFileName );
	}
	for( i=0; i<m_aLastSearchStringList.count(); i++ )
	{
		char sBuffer[512];
		QString sText = *(m_aLastSearchStringList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sLastSearchString, i );
		aCfg.writeEntry( sBuffer, sText );
	}
	for( i=0; i<m_aLastReplaceStringList.count(); i++ )
	{
		char sBuffer[512];
		QString sText = *(m_aLastReplaceStringList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sLastReplaceString, i );
		aCfg.writeEntry( sBuffer, sText );
	}
	for( i=0; i<m_aLastSearchFilesList.count(); i++ )
	{
		char sBuffer[512];
		QString sText = *(m_aLastSearchFilesList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sLastSearchFiles, i );
		aCfg.writeEntry( sBuffer, sText );
	}
	for( i=0; i<m_aLastSearchDirList.count(); i++ )
	{
		char sBuffer[512];
		QString sText = *(m_aLastSearchDirList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sLastSearchDirs, i );
		aCfg.writeEntry( sBuffer, sText );
	}
	for( i=0; i<m_aLastScriptArgumentsList.count(); i++ )
	{
		char sBuffer[512];
		QString sText = *(m_aLastScriptArgumentsList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sLastScriptArguments, i );
		aCfg.writeEntry( sBuffer, sText );
	}
	for( i=0; i<m_aExternalProgsList.count(); i++ )
	{
		char sBuffer[512];
		QString sExternalProg = *(m_aExternalProgsList.at( i ));

		sprintf( sBuffer, "%s.%d", c_sExternalProg, i );
		aCfg.writeEntry( sBuffer, sExternalProg );
	}
#ifndef ZAURUS
	aCfg.endGroup();
#endif
}

void VisiScript::closeEvent( QCloseEvent * pCloseEvent )
{
	CheckAllData();

	saveInit();

	pCloseEvent->accept();
}

MyQCodeEdit * VisiScript::GetActMainScript( int * pIndexFound ) const
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
#ifdef ZAURUS
		QWidget * pPage = (QWidget *)m_pTabWidget->queryList("MyQCodeEdit")->at(i);
		QObject * pChild = pPage;
#else
		QWidget * pPage = m_pTabWidget->page( i );
		QObject * pChild = pPage->child( _SCRIPT_NAME );
#endif
		MyQCodeEdit * pScript = (MyQCodeEdit *)pChild;
		if( pScript )
		{
			if( pScript->isMainFile() )
			{
				if( pIndexFound )
				{
					*pIndexFound = i;
				}
				return pScript;
			}
		}
	}

	if( pIndexFound )
	{
		*pIndexFound = -1;
	}
	// if no main file is found, return the actual script !
	return GetActScript();
}

MyQCodeEdit * VisiScript::GetActScript() const
{
	if( GetActDocumentTabPage() )
	{
		QObject * pChild = GetActDocumentTabPage()->child( _SCRIPT_NAME );

		return (MyQCodeEdit *)pChild;
	}
	return 0;
}

MyQMultiLineEdit * VisiScript::GetActOutput() const
{
	QObject * pChild = GetActDocumentTabPage()->child( _OUTPUT_NAME );

	return (MyQMultiLineEdit *)pChild;
}

MyQCodeEdit * VisiScript::GetScriptForFileName( const QString & strFileName, int * pIndex ) const
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
#ifdef ZAURUS
		QWidget * pPage = (QWidget *)m_pTabWidget->queryList("MyQCodeEdit")->at(i);
		QObject * pChild = pPage;
#else
		QWidget * pPage = m_pTabWidget->page( i );
		QObject * pChild = pPage->child( _SCRIPT_NAME );
#endif
		MyQCodeEdit * pScript = (MyQCodeEdit *)pChild;
		if( pScript )
		{
			QString strTemp = pScript->fileName();
			if( strTemp == strFileName ) 
			{
				if( pIndex )
				{
					*pIndex = i;
				}
				return pScript;
			}
		}
	}
	if( pIndex )
	{
		*pIndex = -1;
	}
	return 0;	// not found !
}

MyQCodeEdit * VisiScript::GetScriptFor( int iIndex ) const
{
	if( iIndex>=0 && iIndex<GetDocumentCount() )
	{
#ifdef ZAURUS
		QWidget * pPage = (QWidget *)m_pTabWidget->queryList("MyQCodeEdit")->at(iIndex);
		QObject * pChild = pPage;
#else
		QWidget * pPage = m_pTabWidget->page( iIndex );
		QObject * pChild = pPage->child( _SCRIPT_NAME );
#endif
		return (MyQCodeEdit *)pChild;
	}
	return 0;
}

MyQMultiLineEdit * VisiScript::GetOutputFor( int iIndex ) const
{
	if( iIndex>=0 && iIndex<GetDocumentCount() )
	{
#ifdef ZAURUS
		QWidget * pPage = (QWidget *)m_pTabWidget->queryList("MyQMultiLineEdit")->at(iIndex);
		QObject * pChild = pPage;
#else
		QWidget * pPage = m_pTabWidget->page( iIndex );
		QObject * pChild = pPage->child( _OUTPUT_NAME );
#endif
		return (MyQMultiLineEdit *)pChild;
	}
	return 0;
}

QWidget * VisiScript::GetActDocumentTabPage() const
{
	return m_pTabWidget->currentPage();
}

int VisiScript::GetDocumentCount() const
{
#ifdef ZAURUS
	return m_pTabWidget->queryList("MyQCodeEdit")->count();
#else
	return m_pTabWidget->count();
#endif
}

void VisiScript::AddDocument( int iDocCount, const QString & sFileName )
{
	QString sFileNameTemp = stripPath( sFileName );

	if( iDocCount>=0 )
	{
		sFileNameTemp = sFileNameTemp.sprintf( "%s%d", (const char *)sFileNameTemp, iDocCount );
	}

    QWidget * pTab = new QWidget( m_pTabWidget, "pTab" );
    m_pTabWidget->addTab( pTab, sFileNameTemp );
#ifndef ZAURUS
    m_pTabWidget->setTabToolTip( pTab, sFileNameTemp );
#endif

	QVBoxLayout * pLayout =  new QVBoxLayout( pTab );

#ifdef ZAURUS
    QVBox * pVBox = new QVBox( /*m_pStack*/pTab );
	pVBox->setSpacing( 4 );
	pVBox->setMargin( 2 );
#else
	QSplitter * pVBox = new QSplitter( Qt::Vertical, /*m_pStack*/pTab );
	//pVBox->setSizes();
#endif

	pLayout->addWidget( pVBox );

	QString sDrive, sDir, sFileNameShort, sExt;
	SplitPath( (const char *) sFileNameTemp, sDrive, sDir, sFileNameShort, sExt );
	int iActLanguageID = getLanguageIDFromExtension(sExt);

    MyQCodeEdit * pScript = new MyQCodeEdit( pVBox, _SCRIPT_NAME, m_pTabWidget, pTab, this );
	pScript->setTabWidth( m_iTabSpace );
	pScript->setLexer( 0, iActLanguageID );
	pScript->setMarginLineNumbers( 1, true );
	pScript->setOverwriteMode( (bool)m_iOverwrite );
	pScript->setSelectionBackgroundColor( QColor( 0,0,0 ) );
	pScript->setSelectionForegroundColor( QColor( 255,255,255 ) );
	pScript->setFileName( sFileNameTemp );

	connect( pScript, SIGNAL( cursorPositionChanged(int,int) ), this, SLOT( sltPositionChanged(int,int) ) );

	//TODO min: pScript->setHorizScrollbar( false );
	//pScript->setFolding( QextScintilla::BoxedTreeFoldStyle );

#ifdef ZAURUS
    MyQMultiLineEdit * pOutput = new ZMyQMultiLineEdit( pVBox, _OUTPUT_NAME );
#else
    MyQMultiLineEdit * pOutput = new MyQMultiLineEdit( pVBox, _OUTPUT_NAME );
#endif
    pOutput->setReadOnly( TRUE );
	pOutput->setWordWrap( MyQMultiLineEdit::NoWrap );

	connect( pOutput, SIGNAL( doubleClicked(int,int) ), this, SLOT( sltOutputDoubleClicked(int,int) ) );

	// ** script edit field should be larger than the output edit field
#ifdef ZAURUS
    pVBox->setStretchFactor( pScript, 3 );
#else
	pVBox->setResizeMode( pScript, QSplitter::Stretch );
	pVBox->setResizeMode( pOutput, QSplitter::FollowSizeHint );
#endif
	//sltMaximizeOutput();

	UpdateFlags();

#ifdef ZAURUS
	SetOutputSize( m_iOutputRatio );
#endif

	m_pTabWidget->showPage( pTab );

#ifdef ZAURUS
	QMultiLineEdit::setDefaultTabStop( m_iTabSpace );
#else
	pOutput->setTabStopWidth( m_iTabSpace );
#endif

	// font settings updaten
	UpdateFont( false );
	UpdateLanguage( false, iActLanguageID );
}

void VisiScript::sltCloseAllDocuments()
{
	int iCount = GetDocumentCount();
	// close all but the first document
	for( int i=iCount-1; i>0; i-- )
	{
		m_pTabWidget->setCurrentPage(i);
		sltCloseActDocument();
	}
	// add one empty document
	g_iDocCount = 1;
	AddDocument( g_iDocCount++ );
	// and now close the first document
	m_pTabWidget->setCurrentPage(0);
	sltCloseActDocument();
}

void VisiScript::sltCloseActDocument()
{
	// at least one document must be open !
	if( GetDocumentCount()>1 )
	{
#ifdef _with_thread_search
		if( m_pFindOutputWindow!=0 && GetActOutput()==m_pFindOutputWindow )
		{
			// can not close a window when a find-in-files search is still running
		}
		else
		{
#endif
			if( CheckActData() )
			{
				QWidget * pPage = m_pTabWidget->currentPage();
				m_pTabWidget->removePage( pPage );
				delete pPage;
			}
#ifdef _with_thread_search
		}
#endif
	}
}

void VisiScript::sltClearActDocument()
{
	//CheckActData();	// --> will be done in sltNewFile() / sltClearScript()

	sltNewFile();
}

template <class T1>
void VisiScript::ForAllDocumentScripts( T1 pFcn )
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		(GetScriptFor( i )->*pFcn)();
	}
}

template <class T1, class T2>
void VisiScript::ForAllDocumentScripts( T1 aValue, T2 pFcn )
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		(GetScriptFor( i )->*pFcn)( aValue );
	}
}

template <class T1, class T2, class T3>
void VisiScript::ForAllDocumentScripts( T1 aValue1, T2 aValue2, T3 pFcn )
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		(GetScriptFor( i )->*pFcn)( aValue1, aValue2 );
	}
}

template <class T1>
void VisiScript::ForAllDocumentOutput( T1 pFcn )
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		(GetOutputFor( i )->*pFcn)();
	}
}

template <class T1, class T2>
void VisiScript::ForAllDocumentOutput( T1 aValue, T2 pFcn )
{
	for( int i=0; i<GetDocumentCount(); i++ )
	{
		(GetOutputFor( i )->*pFcn)( aValue );
	}
}

#ifndef ZAURUS

#include <qdragobject.h> 

static bool IsTextFileDrag( const QMimeSource * pEvent, QString & sFileNameOut )
{
	QStrList aStrList;

	if( QUriDrag::decode( pEvent, aStrList ) )
	{
		if( aStrList.count() == 1 )
		{
			const QString sTest = aStrList.at( 0 );
			const char * s = (const char *)sTest;
			sFileNameOut = QUriDrag::uriToLocalFile( s );
			s = (const char *)sFileNameOut;

			return true;
		}
	}

	return false;
}

void VisiScript::dragEnterEvent( QDragEnterEvent * pEvent )
{
	pEvent->accept( QUriDrag::canDecode( pEvent ) );		
}

void VisiScript::dropEvent( QDropEvent * pEvent )
{
	QString sFileName;

	if( IsTextFileDrag( pEvent, sFileName ) )
	{
		DoImportFile( sFileName );
	}
}

#endif
