/**************************************************************************
 *
 *	project				 : VisiScript
 *
 *	copyright            : (C) 2002-2006 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: E:/home/cvsroot/visiscript/visiscript.h,v $
 *
 *  $Revision: 1.12 $
 *
 *	$Log: visiscript.h,v $
 *	Revision 1.12  2006/01/22 13:16:34  Michael
 *	patches for zaurus platform
 *	
 *	Revision 1.11  2006/01/22 12:31:41  Michael
 *	added new configuration flag: automatic load at start
 *	
 *	Revision 1.10  2006/01/12 22:37:59  Michael
 *	new menu items Close all, Run act. Bugfixes, some modal Dialoges will be reused (position and size are restored), inlining all icons in memory (bin2c)
 *	
 *	Revision 1.9  2006/01/10 22:10:07  Michael
 *	some bugfixes and new features implemented. save and reload actual opend files.
 *	
 *	Revision 1.8  2005/12/14 22:46:12  Michael
 *	added support to handle arguments for scripts
 *	
 *	Revision 1.7  2005/12/03 19:00:57  Michael
 *	implemented find-in-files in second thread
 *	
 *	Revision 1.6  2005/12/03 18:52:52  Michael
 *	new menu item Mark as main, bugfix: search for english words "line" and "file" in error messages in output window
 *	
 *	Revision 1.5  2005/11/27 16:34:13  Michael
 *	improved compile support for all platforms
 *	
 *	Revision 1.4  2005/11/20 16:47:57  Michael
 *	updated docu, bugfixes
 *	
 *	Revision 1.3  2005/11/14 16:27:33  Michael
 *	added filebrowser-class from ZFax application
 *	
 *	Revision 1.2  2005/11/07 21:39:31  Michael
 *	implemented new features for next release
 *	
 *	Revision 1.1.1.1  2005/11/06 11:34:15  Michael
 *	initial checkin of version 0.3.2
 * 
 *
 ***************************************************************************/
/*********************************************************************************
 *																				 *
 * This file is part of the VisiScript package (a frontend for script languages) *
 *																				 *
 * Copyright (C) 2002-2005 by Michael Neuroth.									 *
 *                                                                               *
 * This program is free software; you can redistribute it and/or modify			 *
 * it under the terms of the GNU General Public License as published by    		 *
 * the Free Software Foundation; either version 2 of the License, or             *
 * (at your option) any later version.                                           *
 *                                                                               *
 ********************************************************************************/

#ifndef VISISCRIPT_H
#define VISISCRIPT_H

#include <qlist.h>
#ifdef ZAURUS
template <class T1, class T2>
struct QPair
{
    typedef T1 first_type;
    typedef T2 second_type;

    QPair()
	: first( T1() ), second( T2() )
    {}
    QPair( const T1& t1, const T2& t2 )
	: first( t1 ), second( t2 )
    {}

    T1 first;
    T2 second;
};

template <class T1, class T2>
inline bool operator==( const QPair<T1, T2>& x, const QPair<T1, T2>& y )
{
    return x.first == y.first && x.second == y.second;
}

template <class T1, class T2>
inline bool operator<( const QPair<T1, T2>& x, const QPair<T1, T2>& y )
{
    return x.first < y.first ||
	   ( !( y.first < x.first ) && x.second < y.second );
}
#else
#include <qpair.h>
#endif

#include <qmainwindow.h>

// Problems with save-as functionality with opie file-dialog
//#define _with_opie_filedialog

#define _with_zfax_filedialog
#define _with_thread_search

class FileSelector;
class QObject;
class QToolBar;
class QMenuBar;
class QWidgetStack;
class QVBox;
class QMultiLineEdit;
class QTextView;
class QPopupMenu;
class QAction;
class QTabWidget;
class QLabel;
class QComboBox;
class QWidget;
class QProcess;
class QextScintillaAPIs;
class QCustomEvent;
class QDialog;

class MyQMultiLineEdit;		// to invoke the method hasMarkedText()
class MyQCodeEdit;
class FileDialogS;
class FindDlg;
class ReplaceDlg;
class FindInFilesDlgImpl;
class QextScintillaLexer;
class MyFindInFiles;

#include "codeedit.h"

// ************************************************************************

class VisiScript : public QMainWindow
{
    Q_OBJECT

public:
    VisiScript( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    virtual ~VisiScript();

#ifndef ZAURUS
	virtual void dragEnterEvent( QDragEnterEvent * pEvent );
	virtual void dropEvent( QDropEvent * pEvent );
#endif

	virtual bool DoImportFile( const QString & sFileName, int x = 0, int y = 0 );
	
	virtual void customEvent( QCustomEvent * pEvent );

public slots:
    void sltFileNew();
    virtual void sltFileOpen();
    virtual void sltFileSave();
    virtual void sltFileSaveAs();
    void sltFileSaveOutputAs();
    void sltFileImport();
    void sltFileExport();
	void sltFilePrint();
    void sltEditUndo();
    void sltEditRedo();
    void sltEditCut();
    void sltEditCopy();
    void sltEditPaste();
    void sltEditFindText();
    void sltEditFindNext();
    void sltEditReplaceText();
	void sltEditFindInFiles();
	void sltEditGotoLine();
	void sltClearOutput();
	void sltCommentSelection();
	void sltTabWithSelectedTextPressed();
	void sltShiftTabWithSelectedTextPressed();
	void sltUnCommentSelection();
	void sltAddLineToOutput( const QString & sLine );
    void sltClearScript();
	void sltStartInteractiveModus( bool bEnable );
	void sltStopInteractiveModus();
    void sltRun();
	void sltActRun();
    void sltSelectFont();
	//void sltSelectHelvetica();
	//void sltSelectFixed();
	//void sltSelectUnifont();
	//void sltSelectSmall();
	//void sltSelectMedium();
	//void sltSelectLarge();
	void sltZoomIn();
	void sltZoomOut();
	void sltMaximizeOutput();
	void sltMaximizeScript();
	void sltToggleWordWrap();
	void sltToggleOverwrite();
	void sltCodeFolding();
    void sltShowAbout();
	void sltUpdateLastFilesMenu();
	void sltLastFilesMenuActivated( int iIndex );
	void sltViewMarkActAsMain();

	void sltUpdateChilds();

	virtual void show();

	void sltOutputDoubleClicked( int iPara, int iPos ); 
	void sltPositionChanged( int iLine, int iPos );
	void sltReadFromStdout();
	void sltReadFromStderr();
	void sltProcessExited();
	void sltWriteToStdin( const QString & sText );

protected slots:
    void sltNewFile();
    void sltConfiguration();
    virtual void sltShowScript();
    void sltFileAboutToShow();
    void sltEditAboutToShow();
    void sltViewAboutToShow();
    void sltLanguagesAboutToShow();
    void sltLanguageActivated( int iId );
#ifndef _with_opie_filedialog
    void sltDoImport();
    void sltDoExport();
#endif
	void sltCloseAllDocuments();
	void sltCloseActDocument();
	virtual void sltClearActDocument();

protected:
    virtual void closeEvent( QCloseEvent * pCloseEvent );

    void updateCaption( const QString & sName=QString::null );
    virtual void updateFileSelector();
    void loadInit();
    void saveInit();
    bool CheckActData();
	bool CheckAllData();
    void UpdateFont( bool bForAllDocuments );
	void UpdateFont( MyQCodeEdit * pScript, MyQMultiLineEdit * pOutput );
    void UpdateFont( MyQCodeEdit * pScript, MyQMultiLineEdit * pOutput, const QFont & aFont );
    void UpdateFlags();
    void UpdateLanguage( bool bForAllDocuments, int iActLanguageID );
    void UpdateLanguage( MyQCodeEdit * pScript, int iActLanguageID );
    void UpdateLanguageMenu();
    void UpdateScriptOutputRatio();
	void InsertInSelection( bool bDoInsert, const QString & sToken );
    int GetHeightForOutput( int iPercent );
	void PushLastFileName( const QString & sName );
	bool ExistsLastFileName( const QString & sName );
	bool DoExportFile( const QString & sFileName, bool bSaveOutput=false );
	bool DoExportFile( MyQCodeEdit * pScript );
	void SetOutputSize( int iPercentSize );
	void DoRun(bool bUseMainScript);
	MyQCodeEdit * GetActMainScript( int * pIndexFound=0 ) const;
	MyQCodeEdit * GetActScript() const;
	MyQMultiLineEdit * GetActOutput() const;
	MyQCodeEdit * GetScriptFor( int iIndex ) const;
	MyQCodeEdit * GetScriptForFileName( const QString & strFileName, int * pIndex = 0 ) const;
	MyQMultiLineEdit * GetOutputFor( int iIndex ) const;
	QWidget * GetActDocumentTabPage() const;
	int GetDocumentCount() const;
	void AddDocument( int iDocCount, const QString & sFileName = tr( "empty" ) );
	bool doFindFirst( int iLine = -1, int iIndex = -1 );
	void SetAutoCompletionSource( QextScintilla::AutoCompletionSource aSrc, int iThresh, QextScintillaAPIs * pAPIs );
	QString GetStringForTab() const;
	void updateExternalProgs( QComboBox * pCombo );

	template <class T1>
	void ForAllDocumentScripts( T1 pFcn );
	template <class T1, class T2>
	void ForAllDocumentScripts( T1 aValue, T2 pFcn );
	template <class T1, class T2, class T3>
	void ForAllDocumentScripts( T1 aValue1, T2 aValue2, T3 pFcn );

	template <class T1>
	void ForAllDocumentOutput( T1 pFcn );
	template <class T1, class T2>
	void ForAllDocumentOutput( T1 aValue, T2 pFcn );

	// ** data **
#ifndef _with_opie_filedialog
	FileDialogS *		m_pExportFileDlg;
#endif
#ifndef ZAURUS
	QLabel *			m_pStatusBarMsg;
#endif
	QProcess *			m_pProcess;

	QToolBar *			m_pMenu;
	QToolBar *			m_pButtonBar;
	QMenuBar *			m_pMenuBar;			// QPEMenuBar
	QWidgetStack *		m_pStack;
	QTabWidget *		m_pTabWidget;

    QPopupMenu * 		m_pFile;
    QPopupMenu * 		m_pEdit;
	QPopupMenu * 		m_pView;
	QPopupMenu *		m_pLanguages;
    QPopupMenu * 		m_pLastOpendFiles;

	//QAction *			m_pFixedAction;
	//QAction *			m_pHelveticaAction;
	//QAction *			m_pUnifontAction;
	//QAction *			m_pSmallAction;
	//QAction *			m_pMediumAction;
	//QAction *			m_pLargeAction;
	QAction *			m_pMaximizeScript;
	QAction *			m_pMaximizeOutput;
	QAction *			m_pToggleWordWrap;
	QAction *			m_pToggleOverwrite;
	QAction *			m_pCodeFolding;
	QAction *			m_pCopyAction;
	QAction *			m_pCutAction;
	QAction *			m_pPasteAction;
	QAction *			m_pFindInFilesAction;
	QAction *			m_pCommentAction;
	QAction *			m_pUnCommentAction;
	QAction *			m_pMarkAsMain;

	QString				m_sScriptCmd;
	QString				m_sScriptArguments;

	QString				m_sFontName;
	QString				m_sFontStyle;
	int					m_iFontSize;

	int					m_iWordWrap;
	int					m_iOverwrite;		// temp
	int					m_iSpacesForTabs;
	int					m_iShowWhitespaces;
	int					m_iShowLineNumbers;
	int					m_iAutoIndent;
	int					m_iCodeFolding;
	int					m_iTabSpace;
	int					m_iEolMode;
	QString				m_sLastDirectory;
	QString				m_sExeclCmd;
	int					m_iExecuteMode;		// 1 == popen, 2 == fork/execl, 3 == QProcess
	int					m_iShowStartupFileDialog;
	int					m_iOutputRatio;		// for the ratio of Script and output
	QString				m_sFindText;		// temps for find/replace dialog
	QString				m_sReplaceText;
	QString				m_sSearchFiles;	    // for find in files
	QString				m_sSearchDir;		// for find in files
	bool				m_bAutoFileLoad;
	bool				m_bIsReplace;
	bool				m_bRegularExpr;
	bool				m_bWildcard;
	bool				m_bCaseSensitive;
	bool				m_bOnlyWholeWords;
	bool				m_bSearchBackward;
	bool				m_bSearchSubDirectories;
	bool				m_bReplaceAll;
	bool				m_bInteractiveModus;
	bool				m_bChangeActDir;
	bool				m_bAutoSave;
	bool				m_bBraceMatching;
	bool				m_bAutoClearOutput;
	bool				m_bSetUtf8;
	int					m_iActLine;
	int					m_iActPos;
	int					m_iAutoCompletionThresh;
	QString				m_sAutoCompletionFileName;
	QextScintillaAPIs * m_pAutoCompletionAPI;

	int					m_iLastMainScriptIndex;
	int					m_iLastActFileIndex;
	QList< QPair<int,int> >	m_aLastCursorPos;

	QList<QString>		m_aLastFilesList;
	QList<QString>		m_aLastOpendFilesList;
	QList<QString>		m_aExternalProgsList;
	QList<QString>		m_aLastSearchStringList;
	QList<QString>		m_aLastReplaceStringList;
	QList<QString>		m_aLastSearchFilesList;			// for find in files
	QList<QString>		m_aLastSearchDirList;			// for find in files
	QList<QString>		m_aLastScriptArgumentsList;

	const QString		c_strLineTag;
	const QString		c_strFileTag;
	const QString		c_strLineTagOrg;
	const QString		c_strFileTagOrg;

	FindDlg *				m_pFindDlg;
	ReplaceDlg *			m_pReplaceDlg;
	FindInFilesDlgImpl *	m_pFindInFilesDlg;

#ifdef _with_thread_search
	MyFindInFiles *		m_pFindThread;
	MyQMultiLineEdit *	m_pFindOutputWindow;
	int					m_iCount;
	int					m_iFoundFileCount;
	int					m_iTotalFileCount;
	bool				m_bStopFlag;
#endif

#ifndef ZAURUS
	int					m_iLastPosX;
	int					m_iLastPosY;
	int					m_iLastPosDX;
	int					m_iLastPosDY;
#endif
};

const int EXECUTE_MODE_POPEN = 1;
const int EXECUTE_MODE_EXECL = 2;
const int EXECUTE_MODE_QPROCESS = 3;

#endif // VISISCRIPT
