
#include "zvisiscript.h"
#include "SaveAsDlg.h"
#include "filebrowser.h"

#include <qaction.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qwidgetstack.h>
#include <qtabwidget.h>

#include <qpe/qpetoolbar.h>
#include <qpe/fileselector.h>


#ifdef _use_original_file_dialog
static void copyDocLnk( const DocLnk & source, DocLnk & target )
{
	target.setName(source.name());
	target.setFile(source.file());
	target.setLinkFile(source.linkFile());
	target.setComment(source.comment());
	target.setType(source.type());
	target.setCategories(source.categories());
}
#endif

static int g_iDocCount = 0;

ZVisiScript::ZVisiScript( QWidget* parent, const char* name, WFlags fl )
: VisiScript( parent, name, fl ),
  m_pFileSelector( 0 )
{
	m_pGotoBegin = new QAction( tr( "Begin" ), QString::null, CTRL+Key_B, this, 0, TRUE );
    connect( m_pGotoBegin, SIGNAL( activated() ), this, SLOT( sltGotoBegin() ) );
	m_pGotoEnd = new QAction( tr( "End" ), QString::null, CTRL+Key_E, this, 0, TRUE );
    connect( m_pGotoEnd, SIGNAL( activated() ), this, SLOT( sltGotoEnd() ) );

	if( m_bAutoFileLoad && m_aLastOpendFilesList.count()>0 )
	{
		bool bAnyFileOpend = false;
		for( unsigned int i=0; i<m_aLastOpendFilesList.count(); i++ )
		{
			QPair<int,int> * pCursorPos = m_aLastCursorPos.at(i);
			int x = pCursorPos->first;
			int y = pCursorPos->second;
			bool bImportOk = DoImportFile( *(m_aLastOpendFilesList.at(i)), x, y );
			bAnyFileOpend = bAnyFileOpend || bImportOk;
		}
		if( !bAnyFileOpend )
		{
			AddDocument( g_iDocCount++ );
		}
	}
	else
	{
		AddDocument( g_iDocCount++ );
	}

	m_pTabWidget->setCurrentPage( m_iLastActFileIndex );

	updateFileSelector();

	if( m_iShowStartupFileDialog )
	{
	    sltFileOpen();
	}
	else
	{
		sltShowScript();
		updateCaption();
	}
}

ZVisiScript::~ZVisiScript()
{
	delete m_pFileSelector;
}

void ZVisiScript::sltNewFileDoc( const DocLnk & f )
{
#ifdef _use_original_file_dialog
    DocLnk nf = f;
    nf.setType( /*"text/plain"*/m_sScriptExtention );	// Bugfix 30.8.2003
    nf.setName( tr( "empty" ) );

	copyDocLnk( nf, m_aDoc );

	sltFileNew();

    //updateCaption( m_aDoc.name() );
#endif
}

void ZVisiScript::sltOpenFileDoc( const DocLnk & f )
{
#ifdef _use_original_file_dialog
	// ACHTUNG: man MUSS eine deep-copy durchfuehren !!!
	//error:	m_aDoc = f;
	copyDocLnk( f, m_aDoc );

    //sltClearScript();		// not needed for multiple documents !

    FileManager fm;
    QString txt;
    if ( !fm.loadFile( f, txt ) ) {
	// ####### could be a new file
	//qDebug( "Cannot open file" );
	//return;
    }

    sltShowScript();

	AddDocument( -1, m_aDoc.name() );

    GetActScript()->setText( txt );
    GetActScript()->setEdited(FALSE);

	PushLastFileName( m_aDoc.file() );

    //updateCaption( m_aDoc.name() );
#endif
}

void ZVisiScript::sltGotoBegin()
{
	GetActScript()->setCursorPosition( 0, 0 );
}

void ZVisiScript::sltGotoEnd()
{
	MyQCodeEdit * pScript = GetActScript();
	if( pScript )
	{
		pScript->setCursorPosition( pScript->lines(), pScript->lineLength( pScript->lines() ) );
	}
}

void ZVisiScript::sltFileSaveAs()
{
#ifdef _use_original_file_dialog
	SaveAsDlg * pDlg = new SaveAsDlg( this, "saveasdlg", /*bModal*/TRUE );

	pDlg->m_pFileName->setText( tr( "empty" ) );

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		// ok, pressed
		QString sFileName = pDlg->m_pFileName->text();

	    FileManager fm;

		QString s = GetActScript()->text();
		QString sExt = pDlg->m_pType->currentText();

		// create a new Doc-Link for the copy of the actual file
		DocLnk aDoc;
		aDoc.setType( /*"text/plain"*/sExt );
		aDoc.setName( sFileName );
	    if( fm.saveFile( aDoc, s ) )
		{
			// schreiben war ok
			copyDocLnk( aDoc, m_aDoc );
			//updateCaption( m_aDoc.name() );

			GetActScript()->setFileName( m_aDoc.file() );
			GetActScript()->setEdited( false );

			PushLastFileName( m_aDoc.file() );
		}
		else
		{
			// ggf. todo --> fehlermeldung ausgeben !
		}
	}

	delete pDlg;
#else
#ifdef _with_zfax_filedialog
	fileBrowser dlg(this, false);

	if( dlg.exec() && !dlg.selectedFileName.isEmpty() )
	{
		DoExportFile( dlg.selectedFileName, /*bSaveOutput=*/false );
	}
#else
    VisiScript::sltFileExport();
#endif
#endif
}

#ifdef _use_original_file_dialog
bool ZVisiScript::DoImportFile( const QString & sFileName )
{
	if( VisiScript::DoImportFile( sFileName ) )
	{
		// update/clear doc-link !
		DocLnk aDoc;
		//aDoc.setType( "text/plain" );
		//aDoc.setName( *pFileName );
		copyDocLnk( aDoc, m_aDoc );

		return true;
	}
	return false;
}
#endif

void ZVisiScript::sltFileOpen()
{
#ifdef _use_original_file_dialog
    //CheckData();		// not needed for multiple documents !

    m_pMenu->hide();
    m_pButtonBar->hide();
    clearWState (WState_Reserved1 );
    updateFileSelector();
    m_pStack->raiseWidget( m_pFileSelector );
    m_pFileSelector->reread();
    updateCaption();
#else
#ifdef _with_zfax_filedialog
    // use the new (faster) import dialog
	fileBrowser dlg(this, true);

	if( dlg.exec() && !dlg.selectedFileName.isEmpty() )
	{
		DoImportFile( dlg.selectedFileName );
	}
#else
    VisiScript::sltFileImport();
#endif
#endif
}

void ZVisiScript::sltFileSave()
{
#ifdef _use_original_file_dialog
    // TODO --> funktioniert das wirklich, das m_aDoc ist ja gar nicht multi-dokument faehig !!! ???
    FileManager fm;

	QString s = GetActScript()->text();
    if( fm.saveFile( m_aDoc, s ) )
	{
		GetActScript()->setEdited( false );
    }
	else
	{
		// ggf. todo --> fehlermeldung ausgeben !
	}
#else
	VisiScript::sltFileSave();
#endif
}

void ZVisiScript::sltShowScript()
{
#ifdef _use_original_file_dialog
    m_pFileSelector->hide();
    m_pMenu->show();
    m_pButtonBar->show();
#endif
    m_pStack->raiseWidget( (QWidget *)m_pTabWidget );
    GetActScript()->setFocus();
}

void ZVisiScript::updateFileSelector()
{
#ifdef _use_original_file_dialog
    delete m_pFileSelector;

    m_pFileSelector = new FileSelector( m_sScriptExtention, m_pStack, "fileselector" , TRUE, FALSE );
    connect( m_pFileSelector, SIGNAL( closeMe() ), this, SLOT( sltShowScript() ) );
    connect( m_pFileSelector, SIGNAL( newSelected( const DocLnk &) ), this, SLOT( sltNewFileDoc( const DocLnk & ) ) );
    connect( m_pFileSelector, SIGNAL( fileSelected( const DocLnk &) ), this, SLOT( sltOpenFileDoc( const DocLnk & ) ) );
#else
	// nothing needed here !
#endif
}

